/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.util.Vector;
import java.util.zip.*;
import java.awt.*;
import java.io.*;



/**
   ANIMATOR EDITOR
*/

public class AnimatorEdit extends Animator
{

 public int generator = 0;
 Vector images = new Vector();
 VectorInt depths  = new VectorInt();  // Index of branch if one
 static String message = "ok";

  public Image getImage(int i) { return (Image) images.elementAt(i); }
  public Image getImage() { return getImage(index); }

  /**
     Initialisation
  */
  public void clear() { clear(16); }
  public void clear(int size)
  {
   super.clear(size);
   images = new Vector(size);
   depths = new VectorInt(size);
  }


 /** Add any king of image LOCAL */
 protected void add(Object image, String name, int type, int par, int sib, int son, int depth)
 {
  // On rcupre la parent sur le dernier item
  images.addElement(image);
  if(image == null)
    pictures.addElement(null);
  else
    pictures.addElement((Object) ( (new RawEdit((Image) image)).getRaw() ));
  names.addElement((Object) name);
  types.addElement(type);
  parents.addElement(par);
  nexts.addElement(sib);
  childs.addElement(son);
  depths.addElement(depth);
  count++;
 }


  /**
      Construit l'animator
       partir d'un TreeView
  */

 public final void build(DTreeView tree)
  {
   DTreeNode node;
   int type;
   Object image;   // type d'objet, Image, Gif, etc...
   int depth;

   //System.out.println("edit - build");

   if(tree.rootNode == null) return;

   tree.rootNode.depth = 0;
   count = 0;

   //System.out.println("edit - tree count " + tree.count);

   Vector nodesVector = new Vector(tree.count);
   tree.vectorize(tree.rootNode, false, nodesVector);

   int numNodes = nodesVector.size();
   clear(numNodes);

   for (int i = 0; i < numNodes; i++)
   {
    //System.out.println("node " + i);
    node = (DTreeNode) nodesVector.elementAt(i);

    //System.out.println(node.getText());
    if(node.isDirectory) type = BRANCH;
                    else type = IMAGE;
    image = node.dataObject;
    depth = node.depth;

   	int par = -1;
   	if(node.parent != null) par = nodesVector.indexOf(node.parent);
    int sib = 0;
    if(node.sibling != null) sib = nodesVector.indexOf(node.sibling);
    int son = 0;
    if(node.child != null) son = nodesVector.indexOf(node.child);

    //System.out.println("add");
    add(image, node.text, type, par, sib, son, depth);
   }

   System.gc();
  }


  /**
     Construit un Tree View
      partir d'une animation
  */

  static private float test = (float) 3 / (float) 2;


  public static Image resize(Image dataImage)
  {
    int OW = dataImage.getWidth(null);
    int OH = dataImage.getHeight(null);
    int H;
    int W;

    float rapport = (float) OW / (float) OH;

    if(rapport > test)
     {
       W = 24;
       H = (int) ((float) 24 / rapport);
     }
    else
     {
       H = 16;
       W = (int) ((float) 16 * rapport);
     }

    //message =  " " + rapport + " -- " + OW + " x " + OH + " > " + W + " x " + H;
    // NB: SCALE_SMOOTH not implemented in Visual Caf 2.0
    return dataImage.getScaledInstance(W, H, Image.SCALE_DEFAULT);
  }


  /***************************************

     Build a DTreeView from an animator

  */

  DTreeNode parentNode;
  DTreeNode previousNode;
  DTreeView tree;
  Image expanded  = DirIcons.Expanded;
  Image collapsed = DirIcons.IconDir;


 /** Ajoute un node */
 void addNode(DTreeView tree, DTreeNode newnode, DTreeNode parent, boolean asChild)
 {
  // Premire entre
  if(parent == null && previousNode == null)
  {
    tree.append(newnode);
    return;
  }

  // Si option asChild ajouter comme fils
  if(asChild)
  {
   if(!parent.isDirectory) parent = newnode.parent;
   tree.addChild(newnode, parent);

   //if(par.isExpandable() && !par.isExpanded()) par.expand();
   return;
  }

  tree.addSibling(newnode, previousNode, true);
 }



  /**
     Efface et reconstruit le DTreeView
  */

  public void buildTree(DTreeView newtree)
  {
    tree = newtree;
    tree.clear();

    parentNode = null;
    previousNode = null;
    buildRecurse(0);
  }

  /** Ajoute les branches et images */

  void buildRecurse(int no)
  {
   boolean isChild = true;

   do
   {
    String name = (String) names.elementAt(no);
/*
    System.out.println(no + ") " + name +
    " child "  + childs.elementAt(no) +
    " next "   + nexts.elementAt(no)  +
    " parent " + parents.elementAt(no));
*/
    DTreeNode node = null;
    if(types.elementAt(no) == BRANCH)
    {
     //System.out.println("branch " + name);
     node = new DTreeNode(name, collapsed, expanded, tree);
     node.setDataObject((Object) null);
     node.isDirectory = true;
     node.parent = parentNode;
     addNode(tree, node, parentNode, isChild);

     if(parentNode != null)   parentNode.child = node;
     if(previousNode != null) previousNode.sibling = node;

     int nochild = childs.elementAt(no);
     if(nochild > 0)
     {
      parentNode = node;
      previousNode = null;
      buildRecurse(nochild);
     }
    } // folder
    else
    {
     //System.out.println("image " + name);
/*
     if(parentNode != null)
     System.out.println("parent " + parentNode.toString());
     if(previousNode != null)
     System.out.println("previous " + previousNode.toString());
*/
     try
     {
      Raw raw = (Raw) pictures.elementAt(no);
      Image dataImage = raw.getImage();
      Image treeImage = resize(dataImage);
      node = new DTreeNode(name, treeImage, null, tree);
      node.setDataObject((Object) dataImage);
      node.parent = parentNode;
      addNode(tree, node, parentNode, isChild);
     }
     catch(Exception e) { e.printStackTrace(); }

     if(parentNode != null) parentNode.child = node;
     if(previousNode != null) previousNode.sibling = node;

    } // image/doc

    // Read next item
    //System.out.println("suite, de " + no + " next " + nexts.elementAt(no));

    parentNode = null; // when first passed, always successor
    previousNode = null;

    no = nexts.elementAt(no);
    if(no > 0)
    {
      previousNode = node;
      isChild = false;
    }
   }
   while(no > 0);


  }


  /*
     Cre un nom unique pour une nouvelle entre
     Ajoute 1 ou incrmente le suffixe numrique
  */


  static String extension;
  static String alphanode;
  static String namenode;
  static String namesuffix;
  static int    intsuffix;
  static public void getAlphaNode(String name)
  {
     int i = name.lastIndexOf(".");
     if(i == -1)
     {
      extension = "";
      namenode = name;
     }
     else
     {
      extension = name.substring(i);
      namenode = name.substring(0, i);
     }

     int fin = i;
     i = namenode.length();

     boolean bSuffix = false;
     while(i >= 0)
     {
      if(Character.isDigit(namenode.charAt(i - 1)))
      {
       i--;
       bSuffix = true;
      }
      else break;

     }

     intsuffix = 0;
     namesuffix = "";

     if(bSuffix)
     {
      namesuffix = namenode.substring(i, fin);
      namenode = namenode.substring(0, i);  // prefix alpha
      intsuffix = Integer.valueOf(namesuffix).intValue();
     }

  }

  /**
     EXTERN
     Test if the name already exists in the tree
  */
  static public String getUnic(DTreeView tree, String name)
  {

   while(tree.exists(name))
    {
     getAlphaNode(name);
     name = namenode + Integer.toString(intsuffix + 1) + extension;
    }

   return name;
 }

  /** Save the animator */

  public String getDir() { return dir; }


  public final boolean save(Frame parent, String ndir, String nfic)
  {
    if(ndir == null) ndir = "";
    if(nfic == null)
    {
     FileDialog fd = new FileDialog(parent, "Save animation in Zip archive", FileDialog.SAVE);
     fd.setDirectory(ndir);
     fd.setFile(Path.changeExt("", "aaz"));
     fd.show();

     nfic = fd.getFile();
     if(nfic == null) return false;
     ndir = fd.getDirectory();
    }


    dir = ndir;  // dir permanent de l'animation
    name = nfic; // nom permanent de l'animation
    String path =  Path.merge(dir, name);
    archive(path);
     return true;
  }



 /**
    Archiver une entre
 */


 /**
    Mthode choisie
    Toujours compresse pour maintenant
 */
 static int getMethod()
  {
//    if(!Preferences.ArcCompress) return ZipEntry.STORED;
    return ZipEntry.DEFLATED;
  }


  /**
     levelList
     contruit la liste du contenu d'une branche
     Paramtre: numro du node de la branche
  */
  private int[] levelList(int no)
  {
    no = childs.elementAt(no);
    if(no == 0) return null;  // pas de contenu

    VectorInt vi = new VectorInt(16);
    do
    {
      vi.addElement(no);
      no = nexts.elementAt(no);
    } while(no > 0);

    int liste[] = new int[vi.size()];
    vi.copyInto(liste);

    return liste;
  }

  /**
     Archive rcursivement par niveau de l'arbo
     Param:
     - Liste
     - Stream de l'archive
  */


  private int recurse(String dir, int[] liste, ZipOutputStream zos)
   {
     System.out.println("enter recurse liste len " + liste.length);
     if(liste == null) return 0;
     int counter = 0;

     for(int i = 0; i < liste.length; i++)
     {
      try
      {
      int no = liste[i];
      String entry = (String) names.elementAt(no); // nom
      if(dir.length() > 0) entry = Path.merge(dir, entry);
      System.out.println(i + ") recurse - next name " + entry);
      Raw raw  = (Raw) pictures.elementAt(no);  // image
      int type     = types.elementAt(no); // type

      ZipEntry ze = null;
      boolean result = true;
      try
      {
       System.out.println("entry name " + entry);
       ze = new ZipEntry(entry);
      }   // Dfinition entre
      catch(NullPointerException npe)
      {
        message = entry + " forms bad zip entry ";
        result = false;
        System.out.println(message);
      }
      if(!result) return 0;

      if(type == BRANCH)
      {
        System.out.println(entry + " is BRANCH no=" + no);
        counter += recurse(entry, levelList(no), zos);
      }
      else
      {
       byte[] buffer = raw.getBytes(); // buffer

       //ze.setSize((long) buffer.length);
       try
       {
        System.out.println("put next entry size " + buffer.length);
        zos.putNextEntry(ze);       // Ajoute une entre
        zos.write(buffer, 0, buffer.length);
        zos.closeEntry();
        zos.flush();
        counter++;
       }
       catch(NullPointerException n) { n.printStackTrace(); }
       catch(ZipException z) { z.printStackTrace(); }

      } // image
     } catch(Exception e) { message = e.toString(); System.out.println(message); }
    } // entre suivante

   return counter;
} // recurse


 /**
     Archive une liste de fichiers
     rcursivement ou non selon option
     Param:
     - Rpertoire
     - Fichiers
     - Archive avec path
 */

 public int archive(String archive)
 {
   System.out.println("archive " + archive);
   if(count == 0) return 0;

   File dst = new File(archive);
   int counter = 0;
   try
   {
     FileOutputStream fos = new FileOutputStream(dst); // Output stream de l'archive
     ZipOutputStream zos = new ZipOutputStream(fos);  // Output stream surcharg en Zip
     zos.setMethod(getMethod()); // Mthode

     VectorInt base = new VectorInt(count);
     int no = 0;

     do
     {
      base.addElement(no);
      no = nexts.elementAt(no);
     } while(no > 0);

     int liste[] = new int[base.size()];
     base.copyInto(liste);

     counter += recurse("", liste, zos);
     zos.close();
     fos.close();
   }
   catch(IOException ioe) { message = ioe.toString(); }
   catch(Exception e) { e.printStackTrace(); }

   return counter;
 }


   int findDepth(int no)
   {
    int parent = no;
    int nparent = 0;
    int d = 0;

    do
    {
     try
     {
      d++;
      nparent = parents.elementAt(parent);
      //System.out.println("no " + parent + " parent " + nparent);
      if(nparent < parent) parent = nparent;
      else nparent = -1;
     }
     catch(ArrayIndexOutOfBoundsException a) { a.printStackTrace(); }

    } while(nparent != -1);

    return d;
   }


   /** String reprsentation */

   /**
       Prpare l'arbo sous forme de texte
       Prcd de build() si TreeView dja construite
   */

   public String toString()
	{
	//System.out.println("AEDit tostr - count = " + count);
	if (count == 0) return null;

    String text = "";
    String debug = "";

	for (int i = 0; i < count; i++)
	{
     //System.out.println(i + "/" + count);

     int depth = findDepth(i);
     //System.out.println("depth = " + depth);

     String sBlanks = "";
     if(depth > 0)
     {
 	  StringBuffer blanks = new StringBuffer(depth);
	  for (int n = 0; n < depth; n++) blanks.append(' ');
	  sBlanks = new String(blanks);
	 }

	 if(true)
	 {
	  int parent = parents.elementAt(i);
	  int son = childs.elementAt(i);
	  int sib = nexts.elementAt(i);
	  debug = "[ " + i + " parent " + parent + " child " + son + " next " + sib + " ] ";
     }

	 text += sBlanks + names.elementAt(i) + debug + "\n";
     System.out.println(text);
	}


    return text;
  }




 }


   /*
   static int index;
   static int depth;

   static void getLevel(int no)
   {
    if(no == index) return;

    depth++;

    while(nexts.elementAt(no++) != 0)
    {
     int son = childs.elementAt(no);
     if(son != 0) getLevel(son);
    }

    return;
   }
   */

 /** Insert an item LOCAL */
/*
 protected void insert(int index, Object image, String name, int type, int par, int sib, int son)
 {
  if(sib == 0) add(image, name, type, par, sib, son);

  images.insertElementAt(image, index);
  names.insertElementAt((Object) name, index);
  types.insertElementAt(type, index);
  parents.insertElementAt(par, index);
  nexts.insertElementAt(sib, index);
  childs.insertElementAt(son, index);
  count++;
 }

*/
 /** Insert an IMAGE item, EXPORT */
 /*
 static public void insert(int index, Object image, String name, int type)
 {
  insert(index, image, name, type, false);
 }
*/
 /** Insert a BRANCH or IMAGE item, EXPORT */
/*
 public void insert(int index, Object image, String name, int type, boolean isChild)
 {
  int par;

  if(isChild)
   par = index;    // parent est l'object slect
  else
   par = parents.elementAt(index);  // parent de l'objet slect

  int sib = nexts.elementAt(index);
  int son = 0;  // pas de fils encore
  if(sib == 0) // pas de successeur, ajouter simplement
   add(image, name, type, par, sib, son);

  insert(index, image, name, type, par, sib + 1, son);
 }



 public void insertBranch(int index, String branch)
 {
  int par = parents.elementAt(index);
  int son = childs.elementAt(par);
  int sib = nexts.elementAt(par);

  insert(index, null, branch, BRANCH, par, sib, son);

  // la branche parente change de child
  if(son != 0)
  {
   childs.setElementAt(parents.elementAt(index), index);
  }

  // si on ajoute cette branche  une autre
  if(sib != 0)
  {
   nexts.setElementAt(index + 1, index);
  }


 }

 public void remove(int index)
 {
  if(count > 0)
  {
   images.removeElementAt(index);
   names.removeElementAt(index);
   types.removeElementAt(index);
   nexts.removeElementAt(index);
   childs.removeElementAt(index);
   parents.removeElementAt(index);
   count--;
  }
 }
*/


/*
 public boolean archiveItem(int no, String arcname)
 {
   System.out.println("archive item");
   File dst = new File(arcname);
   ZipEntry ze = null;

   String fname = (String) names.elementAt(no);
   Raw raw = (Raw) pictures.elementAt(no);

   boolean result = true;
   String rp = fname;


   try { ze = new ZipEntry(rp); }   // Dfinition entre
   catch(NullPointerException npe)
   {
     message = rp + " forms bad zip entry " + ze.getName();
     result = false;
   }
   if(!result) return false;

   try
   {
     FileOutputStream fos = new FileOutputStream(dst);// Output stream de l'archive
     ZipOutputStream zos = new ZipOutputStream(fos);// Output stream surcharg en Zip

     zos.putNextEntry(ze);       // Ajoute une entre
     zos.setMethod(getMethod()); // Mthode sur stream

     System.out.println("anim edit save - get bytes");
     byte[] buffer = raw.getBytes(); // buffer
     zos.write(buffer, 0, buffer.length);

     zos.closeEntry();
     zos.flush();
     zos.close();
     fos.close();

     result = true;

   } catch(IOException ioe) { message = ioe.toString(); result = false;}

   return result;
 }
*/
