/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

/**
   FadeFilter is an RGBFilter that uses the
   canFilterIndexColorModel attribute to optimize
   the filtering of images that are based
   on an index colour model.
 */

import java.lang.String;
import java.net.MalformedURLException;
import java.net.URL;
import java.awt.*;

import java.awt.event.*;
import java.applet.Applet;
import java.applet.AppletContext;
import java.io.*;
import java.net.*;
import java.awt.image.*;

/**
   Cette classe dfinit la fonction de filtrage filterRGB
   qui modifie l'intensit des composantes RGB d'un pixel
   et est applique sans doute  chaque pixel
   paramtr par sa position x,y

   Mais si le drapeau canFilterIndexColorModel est vrai
   la filtre s'applique sur la palette et non sur chaque pixel
*/

public class FadeFilter extends RGBImageFilter
{
	// The light level (0 = completely dark, 255 = completely lit)
	private int lightLevel = 255;

	/**
	 * The constructor.
	 */
	public FadeFilter() {	canFilterIndexColorModel = true; }

	/**
	 * Filter a colour value. Hopefully this will only be called
	 * for each colour in the palette, rather than each pixel in
	 * the image.
	 * @returns		The colour rgb, darkened by lightLevel
	 */

	public int filterRGB(int x, int y, int rgb)
  {
		return (
			(rgb & 0xff000000)
			| ((((rgb & 0xff0000) >> 16) * lightLevel / 255) << 16)
			| ((((rgb & 0xff00) >> 8) * lightLevel / 255) << 8)
			| ((rgb & 0xff) * lightLevel / 255)
		);
	}

	/** Getter method for lightlevel */
	public int getLightLevel() { return lightLevel;	}

	/** The setter method for level	 */
	public void setLightLevel(int newLightLevel)
  {
		// Ignore new value unless 0 - 255
		if ((newLightLevel < 256) && (newLightLevel > -1))
	 		this.lightLevel = newLightLevel;
	}
}
