/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.Color;
import java.awt.Graphics;

/**
    Palette shaded

    Pour avoir l'opacit, il faut dfinir le
    colormodel dans le constructeur palette

    Pour spcifier qu'une couleur est transparente
    il faut ajouter son index dans le colormodel,
    ventuellement hriter de palette shaded
    avec un constructeur qui spcifie le nouveau
    color model:
    colorModel = new IndexColorModel(8, r.length, r, g, b, a, transp);
*/

public class PaletteShaded extends Palette
{

 static final int inc1 = 4;
 static final int inc2 = 8;
 static final int inc3 = 12;
 static final int inc4 = 16;
 static final int increment = 2;
/*
 static final int inc1 = 6;
 static final int inc2 = 10;
 static final int inc3 = 14;
 static final int inc4 = 18;
 static final int increment = 2;
*/

 public PaletteShaded()
  {
   super();  // Call fill and set color model
   //{{INIT_CONTROLS
		//}}
}

/**
   Subroutine to shade a row of 16 colors of the palette
*/

public void shade(int row, int rim, int gim, int bim)
{
 int i;
 int ri, bi, gi;
 int rif = 0, bif = 0, gif = 0;

 row  *= 16;

// rim+=8; gim+=8; bim+=8;

 ri = 0;
 bi = 0;
 gi = 0;


 for(i = 0; i < 16; i++)
 {
   if(i == 1) { ri = 64; gi = 64; bi = 64; }
   if(i > 11) { ri+=4; gi+=4; bi+=4; }

   ri += rim; if(ri > 255) ri = 255;
   gi += gim; if(gi > 255) gi = 255;
   bi += bim; if(bi > 255) bi = 255;

   if(i > 6)
   {
    if(rif > 0) {  rim++;  if(rim > increment) rim = increment; }
    if(bif > 0) {  bim++;  if(gim > increment) gim = increment; }
    if(gif > 0) {  gim++;  if(bim > increment) bim = increment; }
   }

   r[row + i] = (byte) ri;
   g[row + i] = (byte) gi;
   b[row + i] = (byte) bi;
 }
}


void fillRow(int row, int i, int ri, int gi, int bi)
{
  fillRow(row, i, (byte) ri, (byte) gi, (byte) bi);
}


void fillRow(int row, int i, byte ri, byte gi, byte bi)
{
  row *= 16;
  r[row + i] = ri;
  g[row + i] = gi;
  b[row + i] = bi;
}


/** Fill the 16 first colors with default */
void fillFirstRow()
{
  changeColor(0,  new Color(0, 0, 0));   // noir
  changeColor(1,  new Color(0, 0, 128));  // bleu
  changeColor(2,  new Color(0, 128, 0));    // vert
  changeColor(3,  new Color(0, 128, 128)); // cyan
  changeColor(4,  new Color(128, 0, 0));   // rouge
  changeColor(5,  new Color(128, 0, 128)); // magenta
  changeColor(6,  new Color(128, 128, 0)); // brown
  changeColor(7,  new Color(192, 192, 192));  // gris
  changeColor(8,  new Color(128, 128, 128));  // gris
  changeColor(9,  new Color(0, 0, 255));    // bleu
  changeColor(10, new Color(0, 255, 0));     // vert
  changeColor(11, new Color(0, 255, 255)); //cyan
  changeColor(12, new Color(255, 0, 0));     // rouge
  changeColor(13, new Color(255, 0, 255)); // magenta
  changeColor(14, new Color(255, 255, 0));  // jaune
  changeColor(15, new Color(255, 255, 255));   // blanc
}


// Create a shaded palette

public void fill()
{
 // Opacit totale
	for (int i = 0; i < a.length; i++)
   {
     a[i] = (byte) 255;
   }
 // Get 16 standard colors in first row
 fillFirstRow();
 shade(1, inc3, inc3, inc3);  // GRAY/GRIS
 shade(2, 0, 0, inc4);  // BLUE/BLEU
 shade(3, 0, inc4, 0);  // GREEN/VERT
 shade(4, 0, inc4, inc4);  // CYAN/TURQUOISE

 // 4-7
 shade(5, inc4, 0, 0);  // RED/ROUGE
 shade(6, inc4, 0, inc4);  // MAGENTA/MAUVE
 shade(7, inc4, inc2, inc1);  // ORANGE

 // 8-11
 shade(8, inc2, inc1, inc1);  // BROWN/BRUN
 shade(9, inc1, inc2, inc4);  // LIGHT BLUE/BLEU CIEL
 shade(10, inc1, inc2, inc1); // LIGHT GREEN/VERT CLAIR
 shade(11, inc1, inc3, inc2); // GREEN-BLUE/BLEU-VERT

 // 12-15
 shade(12, inc3, inc1, inc1); // RED BROWN/BRUN ROUGE
 shade(13, inc3, inc2, inc4); // VIOLET
 shade(14, inc4, inc4, 0); // YELLOW/JAUNE
 shade(15, inc3, inc4, inc2); // ANOTHER GREEN/AUTRE VERT

}


/**
    Autre palette
*/

public void another()
{
 int i;
 int fcolor   = 16;     // Second row first color
       // Green factor, for future improvment only
 final int  gfac = 1;

 fillFirstRow();

 // Dgrade 16 couleurs de chaque range

 for(i = 0; i < 16; i++)
   {
    // GRAY
    fillRow( 1, i,  i, i, i);
    fillRow( 2, i,  fcolor + i * 2, fcolor + i * 2, fcolor + i * 2);

    //RED
    fillRow( 3, i,  i * 2, 0, 0);
    fillRow( 4, i,  fcolor + i * 2, i * gfac, i);

    // GREEN
    fillRow( 5, i,  0, i * gfac, 0);
    fillRow( 6, i,  i, fcolor * gfac, i);

    // BLUE
    fillRow( 7, i,  0, 0, i);
    fillRow( 8, i,  i, i * gfac, fcolor);

    // PURPLE
    fillRow( 9, i, i,0,i);
    fillRow(10, i, fcolor, i * gfac, fcolor);

    // YELLOW
    fillRow(11,  i, fcolor, i * gfac, 0);
    fillRow(12, i, fcolor, fcolor * gfac, i);

    // CYAN
    fillRow(13, i, 0, i * gfac, i);
    fillRow(14, i, i, fcolor * gfac, fcolor);

    fillRow(15, i, fcolor, 0, i);
 }
}



	//{{DECLARE_CONTROLS
	//}}
}
