/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.Hashtable;

/**
    Define a "Raw" image

    Le transparent peut tre lu dans le header
    ou dans la palette indexe.
*/

public class Raw
{
  // 10 bytes header
  short width;        // Largeur
  short height;       // Hauteur
  byte  bottomToTop;  // Direction
  byte  compression;  // Type de compression
  byte  colorModel;   // Modle de couleur
  byte  palette;      // Numro de palette en modele index
  byte  transparent;  // Numro de couleur transparente en index
  byte  version = 0;  // Version number

  int[] code;

  // Direction,
  // top to bottom = BFALSE,
  // bottom to top = BTRUE
  static final byte BTRUE = (byte) 1;
  static final byte BFALSE = (byte) 0;

  // Compression modes
  static final byte UNCOMPRESSED = (byte) 0;
  static final byte RLE = (byte) 1;

  // Color models
  static final byte DIRECT = (byte) 0;
  static final byte INDEXED = (byte) 1;

  /** Constructor with default value and empty buffer */
  public Raw()
  {
    bottomToTop = BFALSE;
    palette = 0;
    compression = UNCOMPRESSED;
    colorModel = INDEXED;
    transparent = 0;
}

  /** Construct from an array of pixels */

  public Raw(int [] code, int width, int height)
  {
    this();
    this.width = (short) width;
    this.height = (short) height;
    this.code = code;
  }

  /**
     Construct a Raw from an array of bytes
     according to color model
  */
  public Raw(byte data[])
  {
   width  = (short) (unsigned(data[0]) * 256 + unsigned(data[1]));
   height = (short) (unsigned(data[2]) * 256 + unsigned(data[3]));
   bottomToTop = data[4];
   compression = data[5];
   colorModel = data[6];
   palette = data[7];
   transparent = data[8];
   version = data[9];

   int size = width * height;
   //System.out.println("Raw construct - size " + size);
   code = new int[size];
   int i = 0;
   int j = 10;
   while(i < size)
   {
    int c;
    if(colorModel == DIRECT)
    {
     c = (unsigned(data[j])   << 24) |
         (unsigned(data[j+1]) << 16) +
         (unsigned(data[j+2]) << 8 ) +
         unsigned(data[j+3]);
     j += 4;
    }
    else
    {
     c = unsigned(data[j++]);
    }

    code[i++] = c;
   }

   //System.out.println("passed raw");
  }



  /**
      Direction byte false = from top to bottom
                byte true =  from bottom to top

      Default it top to bottom
  */

  public int getPalette()     { return unsigned(palette); }
  public int getModel(     )  { return unsigned(colorModel); }
  public int getTransparent() { return unsigned(transparent); }
  public int getCompression() { return unsigned(compression); }
  public boolean isInversed() { return bottomToTop == BTRUE; }
  public boolean isIndexed()  { return colorModel == INDEXED; }


  // a faire
  public int[] UnRLE()
  {
    int [] buffer = code;


    return buffer;

  }

  /**
     Retrieve the image as an array of pixels
  */

  public int[] getPixels()
  {
    switch(compression)
    {
     case RLE: return UnRLE();
     default:  return code;
    }
  }


 /**
     Return image as an array of signed bytes
 */

 public byte[] getBytes()
 {
  int codesize = (int) width * (int) height;
  if(colorModel == DIRECT) codesize *= 4;
  System.out.println("getBytes enter - code size " + codesize);
  int[]  intbuffer;
  byte[] buffer = new byte[codesize + 10];

  try
  {
   buffer[0] = signed(width / 256);
   buffer[1] = signed(width - buffer[1] * 256);
   buffer[2] = signed(height / 256);
   buffer[3] = signed(height - buffer[3] * 256);
   buffer[4] = bottomToTop;
   buffer[5] = compression;
   buffer[6] = colorModel;
   buffer[7] = palette;
   buffer[8] = transparent;
   buffer[9] = version;

   if(bottomToTop == BTRUE) intbuffer = invert(code, width, height);
   else intbuffer = code;

   if(colorModel == INDEXED)
   {
    for(int i = 0; i < codesize; i++)
     buffer[10 + i] = signed(intbuffer[i]);
   }
   else
   {
    int j = 10;
    int color;
    for(int i = 0; i < codesize; i++)
    {
    color = intbuffer[i];
    buffer[j++] = signed((color >> 24) & 255);
    buffer[j++] = signed((color >> 16) & 255);
    buffer[j++] = signed((color >> 8) & 255);
    buffer[j++] = signed(color & 255);
   }
  }
  } catch(ArrayIndexOutOfBoundsException a) { a.printStackTrace(); }

  //System.out.println("bytes buffer len " + buffer.length);
  return buffer;
 }


 /**
     Utilitaires
 */

 static int unsigned(byte b)
 {
  if(b < 0) return (int) b + 256;
  else return (int) b;
 }

 static byte signed(int i)
 {
  return (i < 128) ? (byte) i : (byte) (i - 256);
 }

 static int[] invert(int data[], int w, int h)
 {

  int dptr;
  int sptr;
  int dest[] = new int[w * h] ;
  int h2 = h - 1;

  for(int j = 0; j < h; j++)
  {
   sptr = j * w + 6;
   dptr = (h2 - j) * w;
   for(int i = 0; i < w; i++)
   {
    dest[dptr + i] = data[sptr + i];
   }
  }

  return dest;
 }



  /**
     Retrieve the image loaded
  */

  public Image getImage()
  {
   MemoryImageSource mis;

   if(colorModel == INDEXED)
   {
    ColorModel cm = new PaletteShaded().getColorModel();
    mis = new MemoryImageSource(width, height, cm, getPixels(), 0,  width);
   }
   else
    mis = new MemoryImageSource(width, height, getPixels(), 0,  width);

   return Toolkit.getDefaultToolkit().createImage(mis);
  }



  /**
     Transforme une image Raw en mmoire indexe en Image
  */
  public static Image getImage(byte[] rawImage)
  {

   // this code because offset doesn't work in MIS!!!
   int size = rawImage.length;
   byte[]code = new byte[size];
   System.arraycopy(rawImage, 10, code, 0, size -10);

   int width = (int) rawImage[0] + (int) rawImage[1] * 256;
   int height = (int) rawImage[2] + (int) rawImage[3] * 256;


   MemoryImageSource mis;

   PaletteShaded pal = new PaletteShaded();
   pal.setTransparent(unsigned(rawImage[8]));
   ColorModel cm = pal.getColorModel();
   mis = new MemoryImageSource(width, height, cm, code, 0,  width);
 //  mis = new MemoryImageSource(width, height, cm, rawImage, 10,  width, new Hashtable());

   return Toolkit.getDefaultToolkit().createImage(mis);

  }



  /**
     Return size of Raw image
  */
  public int size()
  {
    return code.length + 10;
  }


 /**
     Charge une image
 */
 void load(String path)
 {
  //System.out.println("load " + path);

  try
  {
    DataInputStream p = new DataInputStream(new FileInputStream(path));

    width = p.readShort();
    height = p.readShort();
    //System.out.println("dim " + width +"x" + height);
    bottomToTop = p.readByte();
    compression = p.readByte();
    colorModel = p.readByte();
    palette = p.readByte();
    transparent = p.readByte();
    version = p.readByte();

    int size = (int) width * (int) height;
    //System.out.println("size " + size);
    code = new int[size];

    for(int i = 0; i < size; i++)
    {
     if(colorModel == DIRECT)
            code[i] = p.readInt();
       else code[i] = unsigned(p.readByte());
    }

    if(bottomToTop == BTRUE) code = invert(code, width, height);

    p.close();

   }
  catch(IOException io) { io.printStackTrace();  }
  catch(Exception e) { e.printStackTrace(); }

 }



}
