/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;
import java.lang.Math.*;
import java.awt.*;


/**
    Cette classe effectue une rotation
    d'une image donne sur un cran virtuel donn
*/

public class Rotate extends Component
{

 static VirtualDirect sScreen;
 static VirtualDirect dScreen;
 static int[]texels;
 static int XSize;
 static int YSize;

 /** Constructeur */

 public Rotate(int [] t, int w, int h)
 {
  sScreen = new VirtualDirect(w, h);
  dScreen = new VirtualDirect(w, h);
  texels = t;
  XSize = w;
  YSize = h;

  sScreen.setTexels(texels, XSize, YSize);
  //{{INIT_CONTROLS
		setSize(0,0);
		//}}
}



void clear()
{
  for(int x = 0; x < XSize; x++)
   for(int y = 0; y < XSize; y++)
     dScreen.pixel(x, y, 0xFF000000);

}

/**
    Plaque la texture (un tableau de couleurs)
    xo, yo:   dcalage selon le degr
    dx, dy:   offset dans l'image
    adx :     -dy
    ady :     dx
*/


void texture(int xo, int yo, int dx, int dy, int adx, int ady)
{

  final int xdiv = 400;
  final int ydiv = 400;

  short adx2 = (short) (adx / xdiv);
  short ady2 = (short) (ady / ydiv);

  int xstart = xo;
  int ystart = yo;

  for(int x = 0; x < XSize; x++)
  {
    // position x,y du texel arrondie
    int xp2 = xstart / xdiv;
    int yp2 = ystart / ydiv;


    for(int y = 0; y < YSize; y++ )
     {
      // calc texel position within original
      // use the hi-bytes which is the integer part of the 8.8 fixed pt.

      int ysrc  = yp2 / xdiv;
      int xsrc  = xp2 / ydiv;

      if(xsrc >= 0)
       if(ysrc >= 0)
        {
         while(xsrc >= XSize) xsrc -= XSize;
         while(ysrc >= YSize) ysrc -= YSize;

         int color = sScreen.getPixel(xsrc, ysrc);
         dScreen.pixel(x, y, color);
        }
      //move to next position
      xp2 += adx2;
      yp2 += ady2;
    }

    // calc new start position for vertical line...
    xstart += dx;
    ystart += dy;
  }
}



  public void rotate(int degree)
  {
   final double Fd2r = 3.1415926535 / 512;
   //offset into (original) picture
   int dy = (int) ( Math.sin(degree * Fd2r) * 160000.0);
   int dx = (int) ( Math.cos(degree * Fd2r) * 160000.0);

   int xo = 0, yo = 0;

   texture(xo, yo, dx, dy, -dy, dx);
  }



Point formulaone(int ox, int oy, int angle)
{
 double rangle = - Math.PI * (double) (angle + 1) / (double) 180.0;
 double dx = (double) ox;
 double dy = (double) oy;

 double x = dx * Math.cos(rangle) - dy * Math.sin(rangle);
 double y = dx * Math.sin(rangle) + dy * Math.cos(rangle);

 return new Point((int) x, (int) y);
}



Point cadrage(int w, int h, int angle)
{
 int cad = 0;
 int x = 0, y = 0;

 while(angle >= 360) angle = angle - 360;

 if(angle < 90)       cad = 1;
 else if(angle < 180) cad = 2;
 else if(angle < 270) cad = 3;
 else                 cad = 4;


 Point oo = formulaone(0, 0, angle); // top left
 Point oh = formulaone(0, h, angle); // bottom left
 Point ow = formulaone(w, 0, angle); // top right
 Point wh = formulaone(w, h, angle); // bottom right

 switch(cad)
 {
  // 0-89
  case 1: y = - ow.y;
	      x = - oo.x;
	      break;
  // 90-179
  case 2: y = - Math.min(ow.y, wh.y);
	      x = - Math.min(ow.x, oo.x);
	      break;
  case 3: y = - Math.min(oh.y, wh.y);
	      x = - Math.min(oh.x, wh.x);
	      break;
  case 4: y = - oo.y;
	      x = - oh.x;
	      break;
  default:break;
}

 return new Point(x, y);
}


/* Rotation du sprite */

void rotate4(int angle)
{
 int w = XSize;
 int h = YSize;

 Point rot = cadrage(w, h, angle);

 for(int j = 0; j < h; j++)
  for(int i = 0; i < w; i++)
   {
    int color = sScreen.getPixel(i, j);
    Point p = formulaone(i, j, angle);
    dScreen.pixel(p.x + rot.x, p.y + rot.y, color);
   }
 return;
}




 /**
    Retourne l'image transforme
  */
  public int[] getTexels() { return dScreen.screen; }

  public Image getImage()
  {
   return dScreen.getImage();
  }


//================== nouveau

/*
public void rotate(int angle)
 {
   Point rot = cadrage(XSize, YSize, angle);
   xmove = rot.x;
   ymove = rot.y;

   rotateImage((double) angle * Math.PI / 180.0);
 }

private final static double QUART = (Math.PI / 2.0);
private final static double OCT = (Math.PI / 4.0);

int xmove, ymove;

public void rotateImage(double angle)
{
    sScreen.setTexels(texels, XSize, YSize);

	if (angle > OCT)
	{
		skewY (1);
		skewX (1);
		skewY (1);
		angle -= QUART;
	}

	skewX(Math.tan (angle / 2.0));
	dScreen.copy(sScreen);
	skewY(Math.sin (angle));
	dScreen.copy(sScreen);
	skewX(Math.tan (angle / 2.0));
}


public void skewX (double skewing)
{
 double l;

 int xmax = (XSize * 3) >> 1;
 int ymax = YSize << 1;
 for (int i = 0; i < ymax; i++)
  {
	l = ((double) i * -skewing) + ((double) YSize * skewing);
	sScreen.copyArea(dScreen, 0, i, xmax, 1, (int)l, 0);
	dScreen.tbLine((int) l, i, 0, i);
	dScreen.tbLine((int) l + xmax, i, xmax, i);
  }
}



public void skewY(double skewing)
{

 double l;
 int xmax = (XSize * 3) >> 1;
 int ymax = YSize << 1;

 for (int i = 0; i < xmax; i++)
 {
	l = ((double) i * skewing);
	sScreen.copyArea(dScreen, i, 0, 1, ymax, 0, (int) l);
	dScreen.tbLine(i, (int) l, i, 0);
	dScreen.tbLine(i, (int) l + ymax, i, ymax);
 }
}
*/

	//{{DECLARE_CONTROLS
	//}}
}

