/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.event.*;

public class ScaleDialog extends DDialog implements ActionListener, AdjustmentListener
{
	java.awt.Panel panelScroll;
    ImageViewer canvas;
	Scrollbar hBar;
	Scrollbar vBar;
	Panel viewport;
	java.awt.Panel panelControl;
	java.awt.Button buttonOk;
	java.awt.Button buttonCancel;
	java.awt.Label labelWidth;
	java.awt.Label labelHeight;

	int iWidth = 0;
	int iHeight = 0;
	Image image = null;
	Image model = null;
	boolean accept = false;
	
	public ScaleDialog(Frame parent, boolean modal)
	{
		super(parent, modal);

		setLayout(new BorderLayout(0,0));
		setVisible(false);
		setSize(insets().left + insets().right + 450,insets().top + insets().bottom + 320);
		setFont(new Font("Dialog", Font.PLAIN, 14));

        panelScroll = new java.awt.Panel(new BorderLayout());
		add(panelScroll, BorderLayout.CENTER);

        vBar = new Scrollbar(Scrollbar.VERTICAL, 0, 1, 1, 97); // param provisoires
        vBar.addAdjustmentListener(this);
        panelScroll.add(vBar, BorderLayout.EAST);

        hBar = new Scrollbar(Scrollbar.HORIZONTAL, 0, 1, 1, 129);
        hBar.addAdjustmentListener(this);        
        panelScroll.add(hBar, BorderLayout.SOUTH);

        viewport = new Panel(new GridLayout());
        viewport.setBackground(Color.white);
        panelScroll.add(viewport, BorderLayout.CENTER);

        canvas = new ImageViewer();
        canvas.setBackground(Color.white);
		viewport.add(canvas);

		panelControl = new java.awt.Panel();
		panelControl.setLayout(null);
		panelControl.setBounds(insets().left + 334,insets().top + 0,116,320);
		add(panelControl, BorderLayout.EAST);

		buttonOk = new java.awt.Button();
		buttonOk.setActionCommand("button");
		buttonOk.setLabel("OK");
		buttonOk.setBounds(8,204,100,30);
		buttonOk.addActionListener(this);
		panelControl.add(buttonOk);

		buttonCancel = new java.awt.Button();
		buttonCancel.setActionCommand("button");
		buttonCancel.setLabel("Cancel");
		buttonCancel.setBounds(8,264,100,30);
		buttonCancel.addActionListener(this);		
		panelControl.add(buttonCancel);

		labelWidth = new java.awt.Label();
		labelWidth.setBounds(16,36,96,24);
		panelControl.add(labelWidth);

		labelHeight = new java.awt.Label();
		labelHeight.setBounds(16,68,96,24);
		panelControl.add(labelHeight);

		setTitle("Scale");

			//{{INIT_CONTROLS
		//}}
}

	
	public ScaleDialog(Frame parent, String title, boolean modal)
	{
		this(parent, modal);
		setTitle(title);
	}

	
    public ScaleDialog(Frame parent)
    {
      this(parent, true);   
    }    
	
	
	void setWidth()
	{
	  labelWidth.setText("Width " + iWidth);
	}  

	void setHeight()
	{
	  labelHeight.setText("Height " + iHeight);
	}  

    public void update(Graphics g) { paint(g); }
	
	public void paint(Graphics g)
	{
	 super.paint(g);   

     iWidth = image.getWidth(null);
     iHeight = image.getHeight(null);
	 
	 setWidth();
	 setHeight();
	 
	 hBar.setValue(iWidth);
	 vBar.setValue(iHeight);	 
	 
	 canvas.setImage(image);
	 canvas.repaint();
	}    
	
	public void setImage(Image image)
	{
	 this.image = image;
	 this.model = image;
	 
     MediaTracker tracker = new MediaTracker(this);
     tracker.addImage(image, 0);
     try { tracker.waitForAll(); } catch(InterruptedException e) { }

	}    
	
	
   public Image getImage()
   {
    this.image = model.getScaledInstance(iWidth, iHeight, Image.SCALE_DEFAULT);
    return image;
   } 
   
   
   public void resize()
   {
    getImage();
    repaint();
   } 
   
   
  public void actionPerformed(ActionEvent event)
  {
    Object object = event.getSource();
    
    if(object == buttonCancel)
    {
      image = model;
      accept = false;
      setVisible(false);
    }  
    else
    if(object == buttonOk)
    {
      accept = true;  
      setVisible(false);  
    }    
    
  }  

   
  public void adjustmentValueChanged(AdjustmentEvent event)
  {
   Object object = event.getSource();
   
   if(object == hBar)
   {
    iWidth = hBar.getValue();
    resize();
   } 
   else
   if(object == vBar)
   {
    iHeight = vBar.getValue();
    resize();
   }
   else return;
   
    
  }  
   
	//{{DECLARE_CONTROLS
	//}}
}
