/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.Label;
import java.awt.LayoutManager;
import common.*;

/**
   Barre d'outils ou pour messages
   Panneaux avec des bords de styles diffrents
*/

public class DBar extends Container
{

    // Style
    public static final int NONE = 0;
    public static final int RAISED = 1;
    public static final int LOWERED = 2;
    public static final int LINE = 3;
    public static final int SEPARED = 4;
    protected int style;

    // Couleurs
    protected Color dark;
    protected Color light;
    protected int width;
    protected int height;
    protected Insets insets = new Insets(1,1,1,1);

    /** Constructs a default DBar */

    public DBar() { this(LINE); }

    public DBar(int s)
    {
       super();
       style = s;
    }


    // Dfinir le style

    public void setBevelStyle(int s) { style = s; }

    public void setInsets(Insets in) { insets = in; }

    public Insets getInsets() { return insets; }


/*
    public void addSeparator(int w, int h)
     {
      DSeparator ds = new DSeparator(w, h);
      ds.setBackground(getBackground());
      add(ds);
     }
*/
    public void addSeparator(int w, int h) { add(new DSeparator(w, h)); }
    public void addSpacer(int w, int h) { add(new DBarSpacer(w, h));}
    public void setHeight(int h)  { addSpacer(0, h); }

    public void update(Graphics g) { paint(g); }

    //int ctr = 0;
	public void paint(Graphics g)
	{
	    //Disp.it("bar repaint" + ctr++);

		Dimension dim = getSize();
		this.width = dim.width;
		this.height = dim.height;

        Color bg = getBackground();
        this.light = bg.brighter();
        this.dark  = bg.darker();

        g.setColor(bg);
        int x = insets.left;
        int y = insets.top;
        g.fillRect(x, y, this.width - x - insets.right, this.height - y - insets.bottom);

        drawBorder(g, 0, 0, this.width - 1, this.height - 1);

		super.paint(g); // Place les composants  l'intrieur
	}


	protected void drawBorder(Graphics g, int x, int y, int w, int h)
	{
	  switch(style)
	    {
	     case RAISED :
	       g.setColor(light);
	       g.drawLine(x, y, x + w, y);
	       g.drawLine(x, y, x, y + h);
	       g.setColor(dark);
	       g.drawLine(x, y + h, x + w, y + h);
	       g.drawLine(x + w, y, x + w, y + h);
	       break;
         case LOWERED :
	       g.setColor(dark);
	       g.drawLine(x, y, x + w, y);
	       g.drawLine(x, y, x, y + h);
	       g.setColor(light);
	       g.drawLine(x, y + h, x + w, y + h);
	       g.drawLine(x + w, y, x + w, y + h);
	       break;
         case LINE :
	       g.setColor(getForeground());
	       g.drawRect(x, y, w, h);
	       break;
	     case SEPARED:
	       g.setColor(dark);
	       g.drawLine(x, y, x + w, y);
	       break;
         case NONE : break;
	     default:    break;
	    }
	}

}

