/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;

public class DDialog extends Dialog implements KeyListener, WindowListener
{
  boolean keyEnabled = false;

  public static final boolean ENTER = true;
  public static final boolean ESCAPE = false;
  public boolean choice = false;

  public Thread thread = null;

  Panel panel;


  public DDialog(Frame parent, boolean modal)
   {
      super(parent, modal);
      setBackground(parent.getBackground());
      setResizable(false);
      setLayout(new BorderLayout(0,0));

      panel = new Panel();
      panel.setLayout(null);
      panel.setBackground(getBackground());
      add(panel, BorderLayout.CENTER);

      addWindowListener(this);
      addKeyListener(this);

   }


  public DDialog(Frame parent)
  {
	  this(parent, true);
  }


  public DDialog(Frame parent, String title)
   {
      this(parent, true);
      setTitle(title);
   }


   public void addNotify()
	{
  	    // Record the size of the window prior to calling parents addNotify.
	 Dimension d = getSize();

	super.addNotify();

	if (fComponentsAdjusted) return;

	// Adjust components according to the getInsets
	setBounds(100, 80, getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
	Component components[] = getComponents();
	for (int i = 0; i < components.length; i++)
	{
		Point p = components[i].getLocation();
		p.translate(getInsets().left, getInsets().top);
		components[i].setLocation(p);
	}
	fComponentsAdjusted = true;
   }

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


    /*
       Set visible appelle show dans Dialog et Window
    */
    public void setVisible(boolean b)
    {
      if(b)
      {
        Rectangle bounds = getParent().getBounds();
        Rectangle abounds = getBounds();

        setLocation(bounds.x + (bounds.width - abounds.width) / 2,
                    bounds.y + (bounds.height - abounds.height) / 2);
      }
      super.setVisible(b);
    }


    public void windowOpened(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowActivated(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowClosing(WindowEvent event)
     {
       if (event.getSource() == DDialog.this)
  	{
 	  if(thread != null)
 	  {
 	   if(thread.isAlive()) thread.destroy();
 	   thread = null;
 	  }
 	  setVisible(false);
 	}
     }


  public void keyTyped(java.awt.event.KeyEvent event) {};
  public void keyReleased(java.awt.event.KeyEvent event) {};
  public void keyPressed(java.awt.event.KeyEvent event)
   {

    if(!keyEnabled) return;
    if(event.isControlDown() || event.isAltDown())  return;

    switch(event.getKeyCode())
     {
   	   case java.awt.event.KeyEvent.VK_ENTER:  choice = ENTER;
                setVisible(false);  break;
   	   case java.awt.event.KeyEvent.VK_ESCAPE: choice = ESCAPE;
                setVisible(false); break;
   	   default:
               char c = event.getKeyChar();
               char newc = new Character(c).charValue();
       	       if (c != newc) event.consume();
               event.setKeyChar(newc);
       	       break;
   	 }
   }

   public void setKeyEnabled(boolean val) { keyEnabled = val; }
   public void setThread(Thread t) { thread = t; }
   public boolean accept() { return choice; }

};
