/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.io.*;


/**
   Gre des fichiers de donnes binaires
*/

public class Data
{

 static void dispError(String name, String error)
 {
    System.out.println(name + "  " +  error);
 }

   /**
      Taille d'un fichier
   */   

  static public int getSize(String path)
    {
      int size = 0;

	  try {
       FileInputStream fiS = new FileInputStream(path);
       size = fiS.available();
       fiS.close();
      } catch (IOException e) {}

      return size;
   }



 /* Loading a binary file */

 static public byte[] bLoad(String path)
  {
   int blocksize = 0;
   int offset    = 0;
   int size      = 0;
   
   byte buffer[] = null;

   try
   {
	FileInputStream fiS = new FileInputStream(path);

    // Lecture block 32 k
    int notAtEnd = 1;
    int available = fiS.available();
    size = available;
    buffer = new byte[available];

    while(notAtEnd > 0)
     {
      // Taille a venir
   	  blocksize = Math.min(available, 32768);
      //Disp.it("block " + blocksize + " / " + available);

   	  if(blocksize > 0)
   	   {
         notAtEnd  = fiS.read(buffer, offset, blocksize);
         offset    += blocksize;
         available -= blocksize;
       }
      else notAtEnd = -1;
     }

    fiS.close();
   } 
   catch (IOException e) { dispError(path, "read error"); }

    
    if(buffer.length != size)
     System.out.println("Data load error: size " + buffer.length + " must be " + size);

    return buffer;
  }



  /**
     Sauvegarde
  */

  static public boolean bSave(String name, byte[] buffer)
   {
    boolean res = false;

    try
    {
	 FileOutputStream foS = new FileOutputStream(name);
     foS.write(buffer);
	 foS.close();
     res = true;
    }
    catch (IOException e) { dispError(name, "enable to save"); }

    return res;
   }


 /**
    Charge un buffer d'entiers
 */
 static public int[] iLoad(String path)
  {
    int loaded = 0;
    int buffer[] = null;

    try
    {
	 DataInputStream fiS = new DataInputStream(new FileInputStream(path));

     int available = fiS.available();
     //System.out.println("available= " + available);
     int size = available / 4;
     
     buffer = new int[size];

     //System.out.println("size to load= " + size);

     int i = 0;
     while(i < size)
     {
      try
      {
       buffer[i] = fiS.readInt();
       loaded++;
      }
      catch(IOException ef) { dispError(path, "error in array"); }
      i++;
    }

     fiS.close();
   } catch (IOException e) { dispError(path, "enable to open"); }

    //System.out.println("loaded " + loaded);
    return buffer;
   }

 /**
    Sauve un buffer d'entiers
 */
 static public int iSave(String path, int buffer[])
  {
    int saved = 0;
    int size = buffer.length;

    try
    {
	 DataOutputStream fiS = new DataOutputStream(new FileOutputStream(path));

     for(int i = 0; i < size; i++)
     {
      try
      {
       fiS.writeInt(buffer[i]);
       saved++;
      }
      catch(IOException ef) { dispError(path, "error in array"); }

    }

     fiS.close();
   } catch (IOException e) { dispError(path, "enable to write"); }

    System.out.println("saved " + saved);
    return saved;
   }




}
