/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import java.lang.*;
import java.io.File;
import common.*;

/**
   Classe qui construit un DTP de l'arborescence du disque
 */

public class DirTree
{

  static String SBuffer = "                                ";   // limit  32
  static int Depth = 3;
  static final int LIMIT_MIN = 2;
  static final int LIMIT_MAX = 32;

  VectorString tree = new VectorString();
  int ARBOMAX;
  String startPath;      // La racine, drive sous Windows

  // Normalement avec la racine du disque courant
  public  DirTree()  { ; }

  static public void setDepth(int d)
  {
    if(d < LIMIT_MIN)  d = LIMIT_MIN;
    if(d > LIMIT_MAX) d = LIMIT_MAX;
    Depth = d;
  }

  // Cre l'arborescence dans une chaine
  public String[] makeDirTree(String p)
  {
   ARBOMAX = 0;
   startPath = p ;

   makeTree(p, 0);

   int m = tree.size();
   String t[] = new String[m];
   tree.copyInto(t);

   return t;
  }

  /**
     Fonction rcursive sur une branche
     Cre l'arborescence dans cette branche
  */
  public void makeTree(String branch, int deep)
  {
    File curdir = new File(branch);     // File
    String curlist[] = curdir.list();   // Liste du contenu

    for(int i = 0; i < curlist.length; i++)
    {
     String fic = curlist[i];
     File f = new File(branch, fic);
     if(f.isDirectory())
     {
      ARBOMAX++;
      StringBuffer e = new StringBuffer(deep + fic.length());
      tree.addElement(e.append(SBuffer.substring(0, deep)).append(fic));
      if(deep < (Depth - 1))
        makeTree(Path.merge(branch, fic), deep + 1);
     }
    }
  } // maketree


  // Lecture d'un chemin sur un panel Tree View

  public String buildPath(DTreeView treeDir)
   {
	DTreeNode node = treeDir.getSelectedNode();
	DTreeNode root = treeDir.getRootNode();
	String path = "";

	while(node != null)
	{
	  path = Path.merge(node.getText(), path);
	  node = node.getParent();
	}
    return Path.merge(startPath, path);
  }


/*
    void build(boolean always)
    {
       boolean previous = false;
       boolean newtree  = false;

  	   TreeDisk = Path.SplitDrive(SrcPath);
  	   currentChar = TreeDisk.charAt(0);
  	   currentChar = Character.toUpperCase(currentChar);

  	   switch( currentChar)
  	   {
		  case 'C': previous = okC;  break;
		  case 'D': previous = okD;  break;
		  case 'E': previous = okE;  break;
		  case 'F': previous = okF;  break;
		  case 'G': previous = okG;  break;
		  case 'H': previous = okH;  break;
		  case 'I': previous = okI;  break;
		  default:   break;
       }

	   // Crer un nouveau rpertoire?
	   if(always || (!previous) || (treeDir.getCount() == 0))
		{
		 Message("Building the tree of " + currentChar);
		 oldDepth = Preferences.Depth;
		 newtree = true;
		 switch (currentChar)
		 {
		  case 'C': treeC = new DirTree();
		  	        treeCstruct = treeC.makeDirTree(TreeDisk);
		  	        okC = true;
		  	        break;
		  case 'D': treeD = new DirTree();
		  	        treeDstruct = treeD.makeDirTree(TreeDisk);
		  	        okD = true;
		  	        break;
		  case 'E': treeE = new DirTree();
		  	        treeEstruct = treeE.makeDirTree(TreeDisk);
		  	        okE = true;
		  	        break;
		  case 'F': treeF = new DirTree();
		  	        treeFstruct = treeF.makeDirTree(TreeDisk);
		  	        okF = true;
		  	        break;
		  case 'G': treeG = new DirTree();
		  	        treeGstruct = treeG.makeDirTree(TreeDisk);
		  	        okG = true;
		  	        break;
		  case 'H': treeH = new DirTree();
		  	        treeHstruct = treeH.makeDirTree(TreeDisk);
		  	        okH = true;
		  	        break;
		  case 'I': treeI = new DirTree();
		  	        treeIstruct = treeI.makeDirTree(TreeDisk);
		  	        okI = true;
		  	        break;
		  default:  Disp.it("Error in tree");
		            break;
         } // switch
		}

       // Nouveau lecteur ou non, on affiche l'arbo  partir de la chaine
	    switch (currentChar)
		 {
		  case 'C': treeDir.setTreeStructure( treeCstruct);
		            currentTree = treeC;
		  	        break;
		  case 'D': treeDir.setTreeStructure( treeDstruct);
		            currentTree = treeD;
		  	        break;
		  case 'E': treeDir.setTreeStructure( treeEstruct);
		            currentTree = treeE;
		  	        break;
		  case 'F': treeDir.setTreeStructure( treeFstruct);
		            currentTree = treeF;
		  	        break;
		  case 'G': treeDir.setTreeStructure( treeGstruct);
		            currentTree = treeG;
		  	        break;
		  case 'H': treeDir.setTreeStructure( treeHstruct);
		            currentTree = treeH;
		  	        break;
		  case 'I': treeDir.setTreeStructure( treeIstruct);
		            currentTree = treeI;
		  	        break;
		  default : treeA = new DirTree();
		  	        treeDir.setTreeStructure( treeA.makeDirTree(TreeDisk));
		            currentTree = treeA;
		  	        break;
         } // switch

      String mess = currentChar + " tree made with " + treeDir.getCount() + " directories.   Depth limited to " + Preferences.Depth;
      if(newtree) Message(mess); else dispTip(mess);
     }
*/

}
