/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

/**
   Classe de chargement de fichier
  */

import java.io.*;
import java.util.*;
import java.lang.*;
import java.awt.*;
import common.*;

 public class Disk
{
   static public final int BINARY = 0;
   static public final int TEXT = 1;
   static public final int AREA = 2;

   public Disk() {			//{{INIT_CONTROLS
		//}}
};


   /**
      Taille d'un fichier
   */   
   static public int size(String path)
    {
      int size = 0;

	  try {
       FileInputStream fiS = new FileInputStream(path);
       size = fiS.available();
       fiS.close();
      } catch (IOException e) { e.printStackTrace(); }

      return size;
   }

   // Chargement text en TextArea

 static public int open(String dirname, String fname, TextArea text)
    {
      int blocksize = 0;
      int textsize = 0;
      String path = Path.merge(dirname, fname);

      System.gc();

	  try {
		FileInputStream fiS = new FileInputStream( path );

        // Lecture block 32 k
        int notAtEnd = 1;
        int available = fiS.available();
        if(available > 0)
        {
         StringBuffer buffer = new StringBuffer(available);

         while(notAtEnd > 0)
          {
           // Taille a venir
    	  blocksize = Math.min(available, 32768);
    	  if(blocksize > 0)
    	   {
            textsize  += blocksize;
            available -= blocksize;
    	    byte[] byteBuffer = new byte[blocksize];
    	    notAtEnd = fiS.read(byteBuffer);
            buffer.append(new String(byteBuffer));  
           }
          else notAtEnd = -1;
         }

         text.setText(buffer.toString());
        }
         
         fiS.close();
       } catch (IOException e)
	    { System.out.println("Error reading file: " + path); }

      //System.out.println("loaded " + textsize);
      return textsize;
   }


  /** Charger un texte ligne par ligne */
  static public String loadText(String dir, String filename)
  {
   String path = Path.merge(dir, filename);
   String ch = System.getProperty("line.separator");
   int seplen = ch.length();
   int size = size(path);
   int loaded = 0;

   if(size == 0) return "";

   System.gc(); 
   StringBuffer sbuffer = new StringBuffer(size);
 
   try
   {
     BufferedReader br = new BufferedReader(new FileReader(path));
     String s = "";
     while(s != null)
      {
        s = br.readLine();
        if(s != null)
         { 
           sbuffer.append(s); 
           sbuffer.append(ch);
           loaded = loaded + s.length() + seplen; 
         }
      }

     br.close();
   } catch(IOException e) { System.out.println("Enable to read " + path); }

   return sbuffer.toString();
 }



  /**
     Insertion dans TextArea  travers TextModel
  */
   public static int insert(String dirname, String fname, TextModel text)
    {
     int inserted = 0;
     String path = dirname + File.separator + fname;
     System.gc();

     try
	   {
		 FileInputStream fiS = new FileInputStream( path );
         int size = fiS.available();
         if(size > 0)
           {
            int notAtEnd = 1;
            while(notAtEnd > 0)
            {
    	        // Chargement
             byte[] byteBuffer = new byte[size];
    	     notAtEnd = fiS.read(byteBuffer);
             if(notAtEnd > 0)
              {
                text.insert(new String(byteBuffer)); // Au curseur
                inserted += byteBuffer.length;
                //System.out.println("added " + inserted + " nae=" + notAtEnd);
             } 
            }
         } 

        fiS.close();
       }
	  catch (IOException e) { System.out.print("Error reading file: " + path); }

      return inserted;
   }


  /** 
      Sauvegarde d'une chane  
  */
  
  static public int save(String dirname, String fName, String textbuffer)
   {
     String path = Path.merge(dirname, fName);
     int size = 0;

     try 
     {
	  FileOutputStream foS = new FileOutputStream(path);
	  size = textbuffer.length();
      foS.write(textbuffer.getBytes());
	  foS.close();
	 } catch (IOException e) { System.out.println("Error writing file " + fName); }
  
     return size;
   }



  /** Sauver un texte ligne par ligne */
  static public int saveText(String dir, String filename, String text)
  {
   String path = Path.merge(dir, filename);
   String ch = System.getProperty("line.separator");
   int index = 0;
   int oldindex = 0;
   int saved = 0;

   System.gc();

   try
   {
     FileWriter fw = new FileWriter(path);
     BufferedWriter bw = new BufferedWriter(fw);

     int size = text.length();
     int seplen = ch.length();

     while(index < size)
      {
       boolean addsep = true;
       index = text.indexOf(ch, oldindex);
       if(index == -1) { index = size; addsep = false; }
       String s = text.substring(oldindex, index);
       bw.write(s, 0, s.length());
       if(addsep) { bw.newLine(); saved += seplen; }
       oldindex = index + seplen;
       saved += s.length();
      }

     bw.close();
   } catch(IOException ioe) { System.out.println("Error writing " + path); }

   return saved;
 }


 // Sauve avec BAK et TMP

 static public String changeExtension(String name, String ext)
 {
   int i = name.lastIndexOf(".");

   if(i > 0)
    return name.substring(0, i) + ext;
   return name + ext;
 }


 static public int saveProtect(String PATH, String filename, int mode, String textstr)
  {
    File freal = new File(PATH, filename);   // Fichier rel
    int size = 0;

    // Si existe, verifie qu'on peut crire
    if(freal.exists())
    {
     try { if(!freal.canWrite()) return 0; }
     catch(SecurityException se) { System.out.println("Security exception for " + filename); }
    }

    String NAME_TMP = changeExtension(filename, ".tmp");
    String NAME_BAK = changeExtension(filename, ".bak");

    File ftmp = new File(PATH, NAME_TMP);   // Fichier TMP

    if(ftmp.exists()) ftmp.delete();   // Enlve TMP s'il reste
    switch(mode)
    {
      case BINARY:  size = save(PATH, NAME_TMP, textstr); break;
      case TEXT:    size = saveText(PATH, NAME_TMP, textstr); break;
      case AREA:    size = save(PATH, NAME_TMP, textstr); break;
//    case LINES:   saveLines(PATH, NAME_TMP, chaine); break;
      default: break;
    }

    // Nouveau nom

    if(freal.exists())
     {
      if(Preferences.MakeBak)
       {
        File fbak = new File(PATH, NAME_BAK); // Ancien BAK
        if(fbak.exists()) fbak.delete();    // Existe ou pas
        freal.renameTo(fbak);
       }
      else
       freal.delete();
     }

   ftmp.renameTo(freal);

   return size;
 }



 /** Sauver une partie d'un texte avec protection */

 static public int saveBlock(String dir, String fname, String textstr)
 {
   return saveProtect(dir, fname, TEXT, textstr);
 }


 /**
    Charge une liste de chaines
   */
 static public int loadLines(String dir, String filename, VectorString vlist)
 {
   String path = Path.merge(dir, filename);
   int counter = 0;

   try
   {
    FileReader fr = new FileReader(path);
    BufferedReader br = new BufferedReader(fr);

    String s = "";
    while(s != null)
     {
      try
      {
       s = br.readLine();
       if(s != null)
       {
         vlist.addElement(s);
         counter++;
       }
      } catch(IOException rioe) { rioe.printStackTrace();  s = null; }
     }

     try { br.close(); } catch(IOException ioe) { ioe.printStackTrace(); }

    } catch (FileNotFoundException fnfe) { System.out.println(path + " not found"); ; }

    return counter;

 }


 /** Charge un vecteur de chaines */
 
 static public int loadUnicLines(String dir, String filename, VectorString vlist)
 {
   String path = Path.merge(dir, filename);
   int counter = 0;

   try
   {
    FileReader fr = new FileReader(path);
    BufferedReader br = new BufferedReader(fr);

    String s = "";
    while(s != null)
     {
      try
      {
       s = br.readLine();
       if(s != null)
        if(vlist.addUnicElement(s)) counter++;
      } catch(IOException e) { s = null; }
     }

   try { br.close(); } catch(IOException ioe) { ioe.printStackTrace(); }

  } catch (FileNotFoundException fnfe) { System.out.println(path + " not found"); ; }

  return counter;
 }


 /** Sauver un vecteur de chaines */

  static public int saveLines(String dir, String filename, VectorString vstring)
  {
   int count = 0;
   String path = Path.merge(dir, filename);

   try
   {
     FileWriter fw = new FileWriter(path);
     BufferedWriter bw = new BufferedWriter(fw);

     int size = vstring.size();
     for(int i = 0; i < size; i++)
      {
       String s = vstring.elementAt(i);
       bw.write(s, 0, s.length());
       bw.newLine();
       count++;
      }

     bw.close();
   } catch(IOException ioe) { System.out.println("Error writing " + path); }

   return count;
 }


/*
   static int convertDos(int i)
   {
    char c = (char) i;

    if(c == '') c = 'e';
    else if(c == '') c = 'e';

    return (int) c;
   }
*/


   /**
      Convertit un texte Dos en texte Windows
   */
/*     
   static public int ImportDos(String dirname, String fname, TextArea text)
    {
      int fsize = 0;
      int tsize = 0;
      String path = Path.merge(dirname, fname);

      System.gc();

	  try {
		FileInputStream fiS = new FileInputStream( path );
        int notAtEnd = 1;
   	    fsize = path.length();
  	    StringBuffer byteBuffer = new StringBuffer(fsize);

        while(fsize > 0)
         {
    	   notAtEnd = fiS.read();
           fsize--;
           tsize++;
           if(notAtEnd > 0)
            byteBuffer.append(convertDos(notAtEnd));
         }

         text.setText(new String(byteBuffer));
         fiS.close();
       } catch (IOException e) { System.out.println("Error reading " + path); }

     return tsize;
   }

*/

	
	
	//{{DECLARE_CONTROLS
	//}}
};
