/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Frame;

/**
  This class for copying files and/or directories
  The copy method is static, and thus this class should not be instantiated.
 */

public class FileCopy extends DDialog
{
    public static boolean success;
    public static boolean always = false; // Mis  jour par la boite de dialogue
    public static boolean cancel = false;
    Frame fParent = null;

    public FileCopy(Frame parent)
    {
     super(parent);
     this.fParent = parent;
    }

    public static boolean success() { return success; }


    public int testReplace(File dst)
    {
      //System.out.println("test" + Preferences.AskReplace);
      if(Preferences.AskReplace && !always)
      {

       if(dst.exists())
       {
        DialogCYN ask =
          new DialogCYN(fParent, dst.getName() + " exists, replace?");
        ask.setVisible(true);
        return(ask.answer);
       }
      }
      return DialogCYN.YES;
    }


    public boolean testSame(File src, File dst)
    {
     String source = src.getPath();
     if(source.equalsIgnoreCase(dst.getPath()))
      {
       String name = Path.getFile(source);
       new DialogMessage(fParent, "Can't copy " + name + " over itself").setVisible(true);
       return true;
      }
     return false;
    }

    /** Copie un fichier, toutes vrifications faites
      Paramtre 1:  chemin source avec nom
      Paramtre 2:  chemin target avec nom
    */

    public int copyFile(File fsrc, File fdst)
    {
      cancel = true;
      success = true;

      if(testSame(fsrc, fdst)) return 0;
      if(!always)
      {
       int r = testReplace(fdst);
       if(r == DialogCYN.NO) return 0;
       if(r == DialogCYN.CANCEL) return 0;
      }

      success = false;
      cancel = false;

      FileInputStream source = null;
      FileOutputStream destination = null;
      byte[] buffer;
      int bytes_read;
      int counter = 0;
      int totalread = (int) fsrc.length();
      int totalwrite = 0;

      sourceAction(fsrc.getPath() + "  to  " + fdst.getPath());

      try
      {
          source = new FileInputStream(fsrc);
	  destination = new FileOutputStream(fdst);
	  buffer      = new byte[2048];

	  while(true)
	   {
             bytes_read = source.read(buffer);
             if (bytes_read == -1) break;
		  destination.write(buffer, 0, bytes_read);
	     totalwrite += bytes_read;
	   }
       }
      catch(Exception e) { }


      if (source != null)
      try { source.close(); } catch (IOException e) { ; }

      if (destination != null)
      try { destination.close(); } catch (IOException e) { }

      success = (totalread == totalwrite);
      if (success)
      {
       counter++;
       long lm = fsrc.lastModified();
       if(lm != 0) fdst.setLastModified(lm);
      }
     return counter;
  }

    /**
      Copie un fichier ou un rpertoire dans un rpertoire
      Premier paramtre: chemin avec nom de fichier
      Second paramtre:  un rpertoire
    */

    public int copy(File fsrc, File fdst) throws IOException
    {
      cancel = false;
      int r = copyLoop(fsrc, fdst);
      always = false;
      return r;
    }


    public int copyLoop(File fsrc, File fdst) throws IOException
    {
      String name = fsrc.getName();
      return copyRename(fsrc, fdst, name);
    }


    // Copie un fichier ou un rpertoire sous un nouveau nom
    // fdst est toujours un rpertoire
   public int copyRename(File fsrc, File fdst, String newName) throws IOException
   {
	 cancel = false;
	 int r = copyRLoop(fsrc, fdst, newName);
	 always = false;
	 return r;
   }

   public int copyRLoop(File fsrc, File fdst, String newName) throws IOException
   {
     int counter = 0;

     if(cancel) return counter;
     //System.out.println("dst=" + fdst.getName());
     if(!Path.isDrive(fdst.getName()))
     if(!fdst.isDirectory())  throw new IOException(fsrc + " not found");
     // Make sure the specified source exists and is readable.
     if(!fsrc.exists())  throw new IOException(fsrc + " not found");
     if(!fsrc.canRead()) throw new IOException(fsrc + " is unreadable");

     if(fsrc.isFile())
      {
         if(!fdst.exists()) fdst.mkdirs(); // on cre le rpertoire cible
         fdst = new File(fdst, newName); // concatne le nom de fichier
         counter += copyFile(fsrc, fdst);
      }
     else if (fsrc.isDirectory())
      {
	 String filename, targetfile, sourcefile;
	 String[] files = fsrc.list();

         fdst = new File(fdst, newName);// On concatne le nom du rpertoire
         if(!fdst.exists()) fdst.mkdirs(); // Puis on le cre physiquement

	 for (int i = 0; i < files.length; i++)
	  {
		filename   = files[i];
		sourcefile = fsrc + File.separator + filename;
		targetfile = fdst + File.separator + filename;

		if ((new File(sourcefile)).isDirectory())
                  counter += copyLoop(new File(sourcefile), fdst);
	    	else
		  counter += copyFile(new File(sourcefile), new File(targetfile));

		if(cancel) return counter;	// par copyFile
	  }
      }
     else System.out.println("Source is nothing!");

     return counter;
   }



   /*
	 File.getParent()
	 - returns null when the file is specified without a directory
	   or is in the root directory. This method handles those cases.
	 Params: f the target File to analyze

	 return the parent directory as a File
	*/

	private static File parent(File f)
	{
		String dirname = f.getParent();
		if (dirname == null)
		{
			if (f.isAbsolute()) return new File(File.separator);
			else return new File(System.getProperty("user.dir"));
		}
		return new File(dirname);
	}

    /** Fire an action event to the listeners. */
	ActionListener actionListener = null;
    public String actionCommand;
    public void sourceAction(String texte)
    {
	if (actionListener != null)
        {
        actionCommand = texte;
		actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, texte));
        }
    }

    public void addActionListener(ActionListener l)
    {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }



}
