/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import common.*;

/**
    A Menu class
    that allow dynamic items
    by adding a list of items
*/

public class MenuDynamic extends DMenu implements ActionListener
{
 Vector permanents;
 int maxsize;
 public FileList dynList;


 public MenuDynamic(String text, int size)
 {
  super(text);
  
  permanents = new Vector(8);
  dynList = new FileList(size);
  maxsize = size;
  
  //{{INIT_CONTROLS
  //}}
}

 public MenuDynamic(String text) 
 {
   this(text, 8);  
 }   
 

 public MenuDynamic(String text, Font f, int s)
 {
  this(text, s);  
  font = f;
  setFont(f);
 } 


 public void setSize(int size)
 {
  maxsize = size;
  dynList.setSize(size);
  restore();
 }   

 /** Ajouter item fixe */
 
 public MenuItem add(MenuItem mi)
 {
  //mi.setActionCommand(mi.getLabel());  
  mi.setFont(font);
  permanents.addElement((Object) mi);
  super.add(mi);
  
  return mi;
 }   
 
 /** Ajout item volatile */
 
 public boolean addDynamic(String path, String fic)
 {
  MenuItem mi = new MenuItem(fic);
  mi.setActionCommand(Integer.toString(dynList.index));
  mi.addActionListener(this);
  mi.setFont(font);

//  super.add(mi);
  boolean res = dynList.addUnic(path, fic); // index incrment pour suivant   
  if(res) restore();
  return res;
 }   


 /** Efface liste */
 public void clear()
 {
   removeAll(); 
   dynList.clear();
 }  
 
 
 public void restore()
 {
  removeAll();

  for(int p = 0; p < permanents.size(); p++)
   {
    super.add((MenuItem) permanents.elementAt(p)); 
   } 

  addList(); 
 }   


 public void addList()
 {
  
  for(int d = 0; d < dynList.index; d++)
  {
   MenuItem mi = new MenuItem(dynList.names[d]); 
   mi.setActionCommand(Integer.toString(d));
   mi.addActionListener(this);
   mi.setFont(font);

   super.add(mi);
  }  

 }
 

 public String get(int numero)
 {
  return dynList.get(numero);  
 }
 
 public String getName(int numero)
 {
  return dynList.getName(numero);
 } 


 public boolean already(String dir, String name)
 {
  return dynList.already(dir, name);  
 }   

 public void remove(String dir, String name)
 {
  dynList.remove(dir, name);
  restore();
 }   

 
 public void actionPerformed(ActionEvent e)
 {
   sendSource(e.getActionCommand());
 }   

  /**
   * Adds a listener to receive action events
   */
  ActionListener actionListener = null;
  
  public void addActionListener(ActionListener l)
  {
   actionListener = AWTEventMulticaster.add(actionListener, l);
  }

 
  public void sendSource(String command)
  {
   if(actionListener != null) 
    actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
  }   

	//{{DECLARE_CONTROLS
	//}}
}
