/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

/**
 * This static class identifies the operating system that 
 * a program is running under. <p> It doesn't need to be instancied
 */

public final class OS
{
    private static boolean isWindows = false;    
    private static boolean isWindows95 = false;
    private static boolean isWindowsNT = false;
    private static boolean isMacintosh = false;
    private static boolean isSolaris = false;
    private static boolean isWindowsNN = false;
    private static boolean isLinux = false;
    
    // Utilis pour passer des message  partir des composants
    public static String message = "Nothing";

    static
    {
        String s = System.getProperty("os.name");

        if(s.equals("Windows 2000"))
         {
          isWindowsNT = true;
          isWindows = true;
         }   

        if(s.equals("Windows NT"))
          {
           isWindowsNT = true;
           isWindows = true;
          } 
        else if(s.equals("Windows 95")) 
          {
           isWindows95 = true;
           isWindows = true;
          } 
        else if(s.equals("Windows 98"))  
          {
           isWindows95 = true;
           isWindows = true;
          }  
        else if(s.equalsIgnoreCase("Windows ME"))  
          {
           isWindows95 = true;
           isWindows = true;
          }  
        else if (s.equals("Macintosh") ||
				 s.equals("macos") ||		//Applet Viewer
				 s.equals("Mac OS") ||		//Netscape
				 s.equals("MacOS"))			//Internet Explorer
                 isMacintosh = true;
        else if (s.equals("SunOS") || 
                 s.equals("Solaris")) 
                 isSolaris = true;
        else if(s.equals("Linux")) 
                 isLinux = true;
        else 
          {
            isWindowsNN = true;
            isWindows = true;
          }  
    }

    private OS() 
    { 
        //{{INIT_CONTROLS
		//}}
    }

    /**
     * Returns true if running under the Windows 95 or Windows NT operating system.
     */
    public static boolean isWindows()   { return isWindows; }
    public static boolean isWindows95() { return (isWindows95);}
    public static boolean isWindowsNT() { return (isWindowsNT); }
    public static boolean isMacintosh() { return (isMacintosh); }
    public static boolean isSolaris()   { return (isSolaris);  }
    public static boolean isWindowsNN() { return(isWindowsNN); }
    public static boolean isLinux()     { return (isLinux); }    
    public static boolean isOther()     { return ( !isWindows() && !isMacintosh && !isSolaris); }  
    
    // Copie de tableaux
/*    
   static public void arraycopy(boolean[] src, boolean[] dst, int len)
    {
     for(int i = 0; i < len; i++)  dst[i] = src[i]; 
    }    

   static public void arraycopy(int[] src, int[] dst, int len)
    {
     for(int i = 0; i < len; i++)  dst[i] = src[i]; 
    }    

   static  public void arraycopy(long[] src, long[] dst, int len)
    {
     for(int i = 0; i < len; i++)  dst[i] = src[i]; 
    }    

   static public void arraycopy(String[] src, String[] dst, int len)
    {
     for(int i = 0; i < len; i++)  dst[i] = src[i]; 
    }    
 */
    
    /*
     * System colors with default initial values, overwritten by toolkit if 
     * system values differ and are available.
     */
    public static int[] systemColors = 
    {
        0xFF005C5C,  // desktop = new Color(0,92,92);
        0xFF000080,  // activeCaption = new Color(0,0,128);
        0xFFFFFFFF,  // activeCaptionText = Color.white;
        0xFFC0C0C0,  // activeCaptionBorder = Color.lightGray;
        0xFF808080,  // inactiveCaption = Color.gray;
        0xFFC0C0C0,  // inactiveCaptionText = Color.lightGray;
        0xFFC0C0C0,  // inactiveCaptionBorder = Color.lightGray;
        0xFFFFFFFF,  // window = Color.white;
        0xFF000000,  // windowBorder = Color.black;
        0xFF000000,  // windowText = Color.black;
        0xFFC0C0C0,  // menu = Color.lightGray;
        0xFF000000,  // menuText = Color.black;
        0xFFC0C0C0,  // text = Color.lightGray;
        0xFF000000,  // textText = Color.black;
        0xFF000080,  // textHighlight = new Color(0,0,128);
        0xFFFFFFFF,  // textHighlightText = Color.white;
        0xFF808080,  // textInactiveText = Color.gray;
//        0xFFC0C0C0,  // control = Color.lightGray;
        0xFFa0c0d0,
        0xFF000000,  // controlText = Color.black;
        0xFFFFFFFF,  // controlHighlight = Color.white;
        0xFFE0E0E0,  // controlLtHighlight = new Color(224,224,224);
        0xFF808080,  // controlShadow = Color.gray;
        0xFF000000,  // controlDkShadow = Color.black;
        0xFFE0E0E0,  // scrollbar = new Color(224,224,224);
        0xFFE0E000,  // info = new Color(224,224,0);
        0xFF000000,  // infoText = Color.black;
    };

    
 
	static boolean isFirstDrawnOverSecond()
	{
		String OSName = System.getProperty("os.name");
		boolean  firstOver = true;

		// We should verify and move these into the OS class
        if (OSName.startsWith("S") ||      // SunOS, Solaris
            OSName.startsWith("Kona") ||   // Kona (Mr. Coffee)
            OSName.startsWith("AIX") ||    // AIX
            OSName.startsWith("OSF") )     // OSF
        {
            firstOver = true;
        }
		else
		{
			boolean jVendorIsNetscape = System.getProperty("java.vendor").startsWith("Netscape");
	        // Windows Netscape 3.0bx
	        if(isWindows() && jVendorIsNetscape && System.getProperty("java.version").equals("1.02") )
	            firstOver = true;
			else
			{
			  if(isMacintosh() && jVendorIsNetscape)  firstOver = true;
			  else  firstOver = false;
			}
		}
		return  firstOver;
	}   
	//{{DECLARE_CONTROLS
	//}}
}
