/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import common.*;

/**

   Fonctionnement du parser
   - Pour chaque ligne on note status:
     * blanche
     * dcale
     * ligne de tirets
   - Selon l'analyse en cours on affecte un status  la prcdente:
     * title
     * titre
       si courante blanche et ant blanche
     * sous-titre
       si titre (courant et ante blanche)
       si previous indent
       (on laisse le drapeau indentx sinon on met text)
     * ligne
   - les titres et sous-titres sont recopis dans le sommaire
*/


public class Outline
{
  int width;
  int height;

  FileWriter fw;
  BufferedWriter bw;

  static String g = "\"";

  public String htmlFile = ""; // le fichier html avec chemin
  public String htmlName = ""; // le fichier html
  public String htmlDir  = "";

  String path;
  String name;
  VectorString title;
  VectorString lines;
  VectorString body;
  VectorString summary;


  static final int TEXT = 16;
  static final int INDENT0 = 0;
  static final int INDENT1 = 1;  // aussi sous-titre si titre
  static final int INDENT2 = 2;
  static final int INDENT3 = 3;
  static final int INDENT4 = 4;
  static final int INDENT5 = 5;
  static final int TITLE = 15;
  static final int BLANK = 32;

  int previousStatus;
  int status;
  int count;
  String codeTitle;
  int no; // numro de titre
  boolean fermeture = false;

  /* Constructeur */


  public Outline(String dir, String textfile)
  {
     name = textfile;
     path = Path.merge(dir, textfile);
}



 /** Dfinit le fichier gnr */
 public void setFile(String dir, String name)
 {
   htmlFile = Path.merge(dir, name);
   htmlDir  = dir;
   htmlName = name;
 }

  public int load()
  {
   int count = 0;

   lines = new VectorString(1024, 128);

   try
   {
     BufferedReader br = new BufferedReader(new FileReader(path));
     String s;
     do
      {
       s = br.readLine();
       if(s != null)
        {
          lines.addElement(s);
          count++;
        }
     } while(s != null);


     br.close();
   } catch(IOException e) { System.out.println("Enable to read " + path); }

   return count;
 }




 private void put(String s)
 {
   try
   {
    bw.write(s, 0, s.length());
    bw.newLine();
   }
   catch(IOException i) { System.out.println("Enable to write " + s); }
 }



  /**
     Ajouter la ligne prcdente
  */


 void indent(String s, int indentation, int header, String size)
 {
   // Cration sommaire
   String sIndent = "";

   codeTitle = "h" + Integer.toString(header);
   for(int i = 0; i < indentation; i++) sIndent += "&nbsp;";

   // ajout d'une ligne de sommaire
   String name = "title" + no++;
   String ref =
   sIndent + "<a href=" + g + "#" + name + g + ">"  +
      "<span style=\"font size=" + size +
   ";color=green\">" + s + "</span>" + "</a>";

   summary.addElement(ref);

   // ajout d'une ligne du texte
   if(fermeture) { body.addElement("</pre>"); }
   body.addElement(
        "<" + codeTitle + "><a name=" + g + name + g + "></a>");
   body.addElement(s + "</" + codeTitle + "> <pre>");
   fermeture = true;
 }

  private void addLine(String s)
  {

   if(s == null) return;

   previousStatus = status;

   switch(status)
   {
    // titre du document
    case TITLE:    title.addElement("<center>" + s + "</center>");
                   break;
    case INDENT0:  indent(s, INDENT0, 3, "14pt"); break;
    case INDENT1:  indent(s, INDENT1, 3, "12pt"); break;
    case INDENT2:  indent(s, INDENT2, 4, "11pt"); break;
    case INDENT3:  indent(s, INDENT3, 5, "10pt"); break;
    case INDENT4:  indent(s, INDENT4, 6, "9pt"); break;
    case INDENT5:  indent(s, INDENT5, 6, "9pt"); break;

    case BLANK: body.addElement(s);
                break;
    case TEXT:
    default:    previousStatus = TEXT;
                body.addElement(s);
                break;
   }
 }

 /**
     Teste si une ligne est blanche
 */
 private boolean isBlank(String s)
 {
   int l = s.length();
   for(int i = 0; i < l; i++)
   {
    if(!Character.isSpaceChar(s.charAt(i))) return false;
   }

   return true;
 }

 private boolean isBlank(int index)
 {
   return isBlank(lines.elementAt(index));
 }
 /**
     Parse la ligne courante
 */
 private void parseLine(int index)
 {
   String s = lines.elementAt(index);

   /*
     si
     - dbut du texte
     - courante centre
     - previous indiffrentes
   */
   if(summary.size() == 0)
    if(s.length() > 6)
     if(s.substring(0, 6).equals("      "))
     {
      // crer ligne prcdente
      status = TITLE;
      addLine(s);
      return;
    }

   /*
      Ligne blanche, ajouter...
   */

   if(isBlank(s))
   {
     status = BLANK;
     addLine(s);
     return;
   }


   /*
      Courante non blanche non centre
      si prcdente non blanche,
      alors texte
      sinon
       si suivante non blanche
       alors texte
       sinon titre
   */

     // ligne ordinaire
     // si prcdente non blanche, alors texte
     if(previousStatus != BLANK)
     {
      status = TEXT;
      addLine(s);
      return;
     }

     // Si dernire ligne ou suivante non blanche, texte
     if((index >= lines.size()) ||
       (index < (lines.size() - 1)) && (!isBlank(index + 1)))
     {
      status = TEXT;
      addLine(s);
      return;
     }

    // sommaire, dtermine l'indentation
    int l = Math.min(s.length(), 5);
    int i;
    for(i = 0; i < l; i++)
    {
     if(!Character.isSpaceChar(s.charAt(i))) break;
    }

    status = i;
    addLine(s);

    return;
 }


 private void putTitle(int index)
 {
   put(title.elementAt(index));
   count++;
 }


 private void putSummary(int index)
 {
   put(summary.elementAt(index) + "<br>");
   count++;
 }

 private void putBody(int index)
 {
   put(body.elementAt(index));
   count++;
 }




 public int createHtml()
 {

   if(load() == 0) return 0;

   title = new VectorString();
   summary = new VectorString(count / 10, 32);
   body    = new VectorString(count * 9 / 10, 32);

   try
   {
     fw = new FileWriter(htmlFile);
     bw = new BufferedWriter(fw);
   }
   catch(IOException i) { System.out.println("Enable to make " + htmlFile); }

   no = 0;


   for(int i = 0; i < lines.size(); i++)
    parseLine(i);

   put("<HTML>");
   put("<HEAD>");
   put("<TITLE> Manual </TITLE>");

   put("<STYLE TYPE=" + g + "text/css" + g + " MEDIA=screen >");
   put("<!--");
   put(" BODY { font-family: " + g + "Courrier New" + g + ", Courrier, mono, Monospaced, Arial }");
   put(" TABLE { font-family: " + g + "Courrier New" + g + ", Courrier, mono, Monospaced, Arial  }");
   put(" H3 { font-size: 12pt; color: green }");
   put(" H4 { font-size: 11pt; color: green }");
   put(" H5 { font-size: 10pt; color: green }");
   put(" H6 { font-size: 9pt; color: green }");
   put(" H7 { font-size: 9pt; color: green }");
   put(" PRE { font-family: " + g + "Courrier New" + g + ", Courrier, mono, Monospaced, Arial  }");
   put(" A:link { text-decoration: none }");
   put(" A:visited { text-decoration: none }");
   put(" A:hover { text-decoration: none }");

   put("-->");
   put("</STYLE>");
   put("</HEAD>");

   if(Preferences.pageColor.equalsIgnoreCase("None"))
    put("<BODY>");
   else
    put("<BODY bgcolor=" + Preferences.pageColor + ">");

   for(int h = 0; h < title.size(); h++) putTitle(h);

   put("<br><hr>");
   put("<TABLE align="+g+"center"+g+" width=" + g + "500" + g + ">");
   put("<TR><TD>");

   for(int s = 0; s < summary.size(); s++) putSummary(s);

   put("</TD></TR>");
   put("</TABLE>");
   put("<HR><br>");

   for(int b = 0; b < body.size(); b++) putBody(b);

   if(fermeture) put("</PRE>");
   put("</BODY>");
   put("</HTML>");

   try { bw.close(); }
   catch(IOException ic) { ic.printStackTrace(); }

   return count;
 }
	//{{DECLARE_CONTROLS
	//}}
}
