/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.*;
import java.io.*;
import java.lang.Math.*;

import common.OS;

// FONCTIONS STATIQUES DE GESTION DE CHEMIN

public final class Path
{
  // Path courant
  static public String getCurDir()  { return System.getProperty("user.dir"); }
  static public String getUserDir() { return getCurDir(); }
  static public String getCurPath() { return getCurDir(); }
  static public String getSysPath() { return getCurDir(); }

  // retourne le drive courant
  static public String getCurDrive()
  {
    if(!OS.isWindows()) return "/";
    return splitDrive(System.getProperty("user.dir"));
  }

  // Nom a-t-il une extension?
  static public boolean hasExtension(String fname)
  {
    return (fname.indexOf('.') != -1);
  }

  // Retourne l'extension
  static public String getExtension(String path)
  {
    int pos = path.lastIndexOf('.');
    if(pos == -1) return "";

    return path.substring(pos + 1);
  }

  /** Remplace l'extension */
  static public String changeExt(String path, String ext)
  {
    int pos = path.lastIndexOf('.');
    if(pos == -1) return path + "." + ext;

    return path.substring(0, pos + 1) + ext;
  }

  // Retourne le chemin sans extension
  static public String removeExt(String path)
  {
    int pos = path.lastIndexOf('.');
    if(pos != -1)
     path = path.substring(0, pos);
    return path;
  }

  /** Test if name is a drive name */
  static public boolean isDrive(String sa)
  {
   if(!OS.isWindows()) return false;
   if (sa == null) return false;
   if (sa.length() == 0) return false;
   if(sa.endsWith(File.separator)) sa = noSuffixed(sa);
   return (sa.charAt(1) == ':') && (sa.length() == 2);
  }


  public static boolean isDirectory(String path)
  {
   return (new File(path)).isDirectory();
  }

  // Concatnation

  static public String merge(String path, String nom)
  {
    if(nom == null) return path;
    if(path == null) path = "";
    if(path.length() == 0) return nom;
    if(path.endsWith(File.separator)) return path + nom;
    return path + File.separator + nom;
  }

  static public String pastePath(String p, String suf) { return merge(p, suf); }

  static public String merge(String drive, String dir, String nom)
  {
    if(nom == null)   nom = "";
    if(dir == null)   dir = "";
    if(drive == null) drive = "";
    if(!drive.endsWith(File.separator)) drive = drive + File.separator;
    if(!dir.endsWith(File.separator)) dir = dir + File.separator;
    return drive + dir + nom;
  }


// Isole et retourne nom de lecteur

  static public String splitDrive(String path)
  {
     if(!OS.isWindows()) return "/";

     int sep = path.indexOf(":");

     // if -1 or not normal, return empty
     if(sep > 0)  return(path.substring(0, sep + 1));
     else
     {
         return "";
     }
  }

 // Retourne un numro d'unit dans un chemin

 static public int getUnit(String path)
 {
   if(!OS.isWindows()) return 0;
   String drive = splitDrive(path).toLowerCase();
   char   cdrive = drive.charAt(0);
   return (int) cdrive - (int) 'a';
 }

 // Isole le nom de fichier dans un chemin
 public static String splitFile(String path)  { return getFile(path); }
 public static String getFile(String path)
 {
   File f = new File(path);

   if(f.isDirectory()) return ("");  // Pas de nom de fichier

   int fio = path.lastIndexOf(File.separator) + 1;
   int aio = path.lastIndexOf('/') + 1;
   fio = Math.max(fio, aio);
   int dio = path.lastIndexOf(":") + 1;

   fio = fio > dio ? fio : dio;
   if(fio > 0) return path.substring(fio);
   return path;
 }

 /** Extract last name from the string */
 public static String splitLast(String path)
 {
   int fio = path.lastIndexOf(File.separator) + 1; // last separator
   int dio = path.lastIndexOf(":") + 1;           // drive separator
   String last;

   fio = fio > dio ? fio : dio;  // the last of the two ones
   if(fio > 0)
     last = path.substring(fio);
   else
     last = path;

   if(last.equals(File.separator)) last = "";
   return last;
 }


 static public String ADir;   // Chemin absolu
 static public String Drive;
 static public String Dir;    // Chemin relatif
 static public String Filename;
 static public String Node;
 static public String Extension;


 /**
   Dispatch a path holding a filename, into components
   ADir      drive+dir
   Drive
   Dir
   Filename
   Node
   Extension
 */
 public static void split(String fullpath)
 {
   int len = fullpath.length();
   int sep = fullpath.indexOf(":");

   if(sep > 0)
   {
    sep++;   // pointe sur char suivant :
    if(OS.isWindows())
      Drive = fullpath.substring(0, sep);
    else
      Drive = "/";
   }
   else
   {
    sep = 0;
    if(OS.isWindows())
         Drive = "";
    else Drive = "/";
   }

   if(fullpath.charAt(sep) == File.separatorChar) sep++;  // passe "/"

   int lio = fullpath.lastIndexOf(File.separatorChar);
   //Disp.it("now sep " + sep + " lio " + lio);

   if(lio > sep)
   {
    Dir   = fullpath.substring(sep, lio);
    Filename = fullpath.substring(lio + 1, len);
   }
   else
   {
    Dir = "";
    Filename = fullpath.substring(sep, len);
   }

   lio = Filename.indexOf('.');
   if(lio == -1)
   {
    Node = Filename;
    Extension = "";
   }
   else
   {
    Node = Filename.substring(0, lio);
    Extension = Filename.substring(lio + 1);
   }

   // Chemin absolu
   ADir = Drive + File.separator + Dir;
 }

  // return filename"s node, remove extension
  static public String splitNode(String path) { return getNode(path); }
  static public String getNode(String Filename)
  {
   int sio = Filename.lastIndexOf(File.separatorChar);
   if(sio != -1) Filename = Filename.substring(sio + 1);

   int dio = Filename.lastIndexOf('.');
   if(dio != -1) Filename = Filename.substring(0, dio);
   return Filename;
  }



  // Isolate directory + filanme, remove dir or root
 public static String splitRelativePath(String path)
 { return getRelativePath(path); }
 public static String getRelativePath(String path)
 {
   int dio = path.lastIndexOf(":");
   if(dio > 0) dio++;
   else dio = 0;
   if(path.charAt(dio) == File.separatorChar) dio++;

   return path.substring(dio);
 }



 // Isolate disk +  directory (remove filename and separator)
 // Test on disk if directory or file
 public static String splitPath(String fullpath)
 {
    return getPath(fullpath);
 }
 public static String getPath(String fullpath)
 {
   // La chaine se termine par un nom de rpertoire explicite
   if(fullpath.endsWith(File.separator))
   {
     return noSuffixed(fullpath);
   }

   int dio = fullpath.lastIndexOf(File.separator);
   int aio = fullpath.lastIndexOf('*');
   // Teste si la chaine est un rpertoire
   // Si pas de masque
   if(aio == -1)
   {
    File f = new File(fullpath);
    if(f.isDirectory()) return fullpath;
   }

   // Si c'est un fichier
   // Si chemin dans la chaine, le retourne sans le nom de fichier

   if(dio > 0) return fullpath.substring(0, dio);

   // Pas de rpertoire, si drive, le retourne
   dio = fullpath.indexOf(":");
   if(dio > 0)
    return fullpath.substring(0, dio + 1);

   // Ni dir ni drive, retourne rien
   if(OS.isWindows())  return "";
   return "/";

 }


  // Cre une racine de dir, non utilisable avec "File"
  static public String makeWinRoot(String nom)
  {
   if(OS.isWindows())
   {
    if(nom.endsWith(":")) nom = nom + File.separator;
   }
   return nom;
  }

  static public String makeJavaRoot(String nom)
  {
   if(nom.endsWith(File.separator))
   {
     if(nom == "/") return "/";
     return nom.substring(0, nom.length() - 1);
   }
   return nom;
  }

  // Cre paramtre
  static public String noSuffixed(String nom)
  {
   if(nom == "/") return nom;
   if(!nom.endsWith(File.separator)) return nom;
   return nom.substring(0, nom.length() - 1);
  }


 static public String suffixed(String nom)
  {
   if(nom.endsWith(File.separator)) return nom;
   return nom + File.separator;
  }

  /**
    remplace les barres
  */
  static public String setCanonical(String origin)
  {
   int l = origin.length();
   char buf[] = new char[l];

   origin.getChars(0, l, buf, 0);

   // replace for the correct separator
   for(int i = 0; i < l; i++)
   {
    char c = buf[i];
    if(c == '\\' || c == '/')
    {
     c = File.separatorChar;
     buf[i] = c;
    }
   }
   // remove ending separator for windows only
   if(OS.isWindows())
   {
     while (buf[l - 1] == File.separatorChar)  l--;
   }

   return new String(buf, 0, l);
  }



  // Test si format n:    (pour windows, voir autres OS)
  public static boolean isValidDrive(String str)
  {
   if(str == "/") return true;
   if(!OS.isWindows())
      {
        return true;
      }

   if(str.charAt(1) != ':') return false;
   if(str.length() > 2) return false;
   if(!Character.isLetter(str.charAt(0))) return false;
   return true;
  }

 /** Compare two filename */

 public static boolean equals(String src, String dst)
 {
  if(OS.isWindows())
  {
   return src.equalsIgnoreCase(dst);
  }

  return src.equals(dst);
 }


 /**
     Fichiers de mme taille et mme date
 */

 public static boolean sameVersion(String src, String dst)
 {
   File s = new File(src);
   File d = new File(dst);

   //System.out.println("same version? " + src + " " + dst);

   return (s.length() == d.length()) &&
          (s.lastModified() == d.lastModified());
 }

 public static boolean newVersion(String src, String dst)
 {
   File s = new File(src);
   File d = new File(dst);
   return (s.lastModified() > d.lastModified());
 }

}
