/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

/**
   Fonctionnement:
   1) Temporary: crer l'objet Temporary, en paramtre le rpertoire de base
   2) make: cre le rp temporaire, paramtres  rp et nom du fichier  extraire
   3) alreadyHere;  teste si fichier dja prsent dans le rp tempo
      si fichier dja l, stop
   4) extractFile
      sinon extraire le fichier dans le chemin
   5) isCreated:  teste si le fichier  lire est cre.
      Si pas trouv, stop, message erreur
      Sinon, affecte les noms dfinitifs
   6) OpenPath
      lecture, accs fichier
   7) kill sans paramtre

*/
import java.io.File;


public class Temporary
{
 String createdPath;  // chemin complet du fichier  lire
 String createdDir;   // rpertoire temporaire complet
 String createdFile;  // fichier  lire dans le chemin temporaire

 String tomakeFile;  // fichier  crer dans le chemin temporaire
 String tomakeDir;   // rpertoire temporaire complet  crer
 String tomakePath;  // rp temporaire + fichier
 
 String message = "ok";
 boolean result = true;

 static String defaultDir  = System.getProperty("user.dir") + File.separator + "AE$$$.Temporary";
 /** 
    Constructeur
  */
  
 public Temporary(String basedir)
 {
  clear();
  tomakeDir  = basedir + File.separator + "AE$$$.Temporary";
  
  //{{INIT_CONTROLS
		//}}
}   

 public Temporary()
 {
  clear();
  tomakeDir = defaultDir;
 }
 

 public void clear()
 {
  tomakeFile = null;
  tomakeDir  = null;
  tomakePath = null;
  createdPath = null;
  createdDir  = null;
  createdFile = null;
}
 /** Crer le chemin temporaire et dfinir les noms */
 public boolean make(String file)
 {
  // tomakeDir cr par le constructeur  
  tomakeFile = Path.splitFile(file);
  tomakePath = Path.merge(tomakeDir, tomakeFile);
  
  File f = new File(tomakeDir);
  if(!f.exists())
  {
    if(!f.mkdirs()) return false;
  }
  createdDir = tomakeDir;
  return true;
 }

 /** Voir si fichier (et dir) existent dja */
 public boolean alreadyCopied()
 {
  return (new File(tomakeDir, tomakeFile)).exists(); 
 }

 /** Crer le chemin complet avec fichier et vrifier cr */
 public boolean isCreated()
 {
  if((new File(createdDir, tomakeFile)).exists()) 
  {
   createdFile = tomakeFile; 
   createdPath = tomakePath;
   return true;
  }
  return false;
 } 


 /**
   Teste si thread est en cours
   En cours si
   - si le fichier  lire existe et
   - si le fichier  lire est utilis
   Pas en cours si
   - le fichier n'existe pas ou
   - il existe et on peut l'effacer
 */
 
 boolean isRunning()
 {
   File f = new File(createdPath);
   if(!f.exists()) return false;
   if(f.canWrite()) return false;
   return true;
 }   

 /**
    Effacer le rpertoire temporaire et le fichier
 */
 public boolean kill()
 {
  
  boolean result = true;

  if(createdPath != null)
  {
   if(!Deleter.delete(createdPath)) result = false;
  } 
  //Disp.it("file deleted=" + result);
  //Disp.it("in kill, deleted, dir=" + createdDir);
  if(createdDir == null) return false;
  if(Deleter.deleteDir(createdDir) < 1) result = false;
  
  if(!result)
   message = "Temporary files not deleted";
  else 
   message = createdFile + " successfully achieved";
  return result;
 }
 
 
 /**
    Effacement inconditionel 
 */
 public static void clean()
 {
   Deleter.deleteDir(defaultDir);  
 }   

  

  String arcDir;
  String arcName;
  String fileRead;

  public void setNames(String aDir, String aName, String fileToExtract)
  {
    arcName = aName;
    arcDir = aDir;
    fileRead = fileToExtract;
  }  
 
  /** Algo qui extrait et cre le fichier  lire */
  public boolean open()
   {
      // Crer le rpertoire temporaire
      if(!make(fileRead))
       {
         message = "Enable to make the temporary directory...";
         result = false;
         return false;
       }  

      String arc = Path.merge(arcDir, arcName);

      // copier fichier
      /*
      if(alreadyCopied())
       { 
         message = "Already here"; 
         result = false;
         return false;
       }
      */ 

      //Disp.it("extract " + arc + " " + fileRead + " " + createdDir);
      if(!Archiver.extractFile(arc, fileRead, createdDir, false)) 
       { 
         message = Archiver.getMessage(); 
         //Disp.it(message);
         result = false;
         return false;
       }
      //Disp.it("extract =" + isCreated()); 
      // mettre  jour les variables
      if(!isCreated())
      {
        message = "Error when creating temporary file";  
        result = false;
        return false;
      } 
      return true;
   }

  /** Cre le fichier  voir */
  
  public boolean open(String arcDir, String arcName, String fileToExtract)
  {
   setNames(arcDir, arcName, fileToExtract);
   return open();
  }  
  
	//{{DECLARE_CONTROLS
	//}}
}
