/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;

public class TreeManager extends Panel implements ActionListener, ComponentListener, MouseListener
{
 DMessageBar statusbar;
 Frame frame;
 DToolBar toolbar;

 TreeDir treeDir;
 DButton buttonMakeSrc;
 DButton buttonMakeDst;
 DButton buttonBuild;


 public TreeManager(Frame f, DMessageBar dmb)
 {
    frame = f;
    setLayout(new BorderLayout());
    setVisible(false);

    toolbar = new DToolBar();
    add(toolbar, BorderLayout.NORTH);

    treeDir = new TreeDir();
    treeDir.addActionListener(this);
    treeDir.addMouseListener(this);
    add(treeDir, BorderLayout.CENTER);


    buttonMakeSrc = new DButton(Icons.Open, "Source");
    buttonMakeSrc.setSize(84, 24);
    //buttonMakeSrc.setBackground(PrgBackground);
    buttonMakeSrc.setTip("Display selected dir in source window");
    buttonMakeSrc.addActionListener(this);
    toolbar.add(buttonMakeSrc);

    buttonMakeDst = new DButton(Icons.Open, "Target");
    buttonMakeDst.setSize(84, 24);
    //buttonMakeDst.setBackground(PrgBackground);
    buttonMakeDst.setTip("Use the selected dir as target");
    buttonMakeDst.addActionListener(this);
    toolbar.add(buttonMakeDst);


    buttonBuild = new DButton(Icons.Cancel, "Rebuild");
    buttonBuild.setSize(84, 24);
//  buttonBuild.setBackground(PrgBackground);
    buttonBuild.setTip("Reload the tree of dirs");
    buttonBuild.addActionListener(this);
    toolbar.add(buttonBuild);

    //treeDir.init();
    treeDir.setStyle(Preferences.TreeStyle);
 }

 boolean started = false;
 public void init()
  {
    if (started) return;
    started = true;
    treeDir.init();
  }

 public void setStyle(Style s)
  {
    treeDir.setStyle(s);
  }


   public void message(String message)
   {
     dispTip(message);
     // rajouter ici history si besoin
   }

   public void dispTip(String message)
   {
     if(statusbar != null)
      statusbar.setText(" " + message);
   }

   public String getSelected()
   {
    return treeDir.getSelectedPath();
   }

  public void rebuild()
    {
      dispTip("Rebuilding the tree");
      treeDir.clear();
      treeDir.init();
      treeDir.repaint();
      treeDir.requestFocus();
    }



  // save time of last mouse down (for double click)
  //long timeMouseDown = 0;
  //int  doubleClickResolution = 300;	// double-click speed in milliseconds

  public void mousePressed(java.awt.event.MouseEvent event)   {}
  public void mouseReleased(java.awt.event.MouseEvent event) {}
  public void mouseEntered(java.awt.event.MouseEvent event){}
  public void mouseExited(java.awt.event.MouseEvent event){}
  public void mouseClicked(java.awt.event.MouseEvent event)
   {
      Object object = event.getSource();
      int click = event.getClickCount();
      if (click == 2)
       {
         sendAction("src");
       }

          // Souris clique deux fois
          /*
  		  if ((event.getWhen() - timeMouseDown ) < doubleClickResolution)
     	  {
			 sendAction("src");
	  	  }
		  else //single click action could be added here
          {
             timeMouseDown = event.getWhen();
          }
          */
     }



   public void actionPerformed(java.awt.event.ActionEvent event)
   {
	Object object = event.getSource();

	if (object == buttonMakeSrc)        sendAction("src");
	else if (object == buttonMakeDst)   sendAction("dst");
	else if (object == buttonBuild)     rebuild();
	else if (object == treeDir)  // enter
	 {
	    String command = event.getActionCommand();
	    if (command.equalsIgnoreCase("enter")) sendAction("src");
	 }

   }


  transient ActionListener actionListener;

  /**
    * Adds the specified action listener to receive action events
    * from this button.
    */
  public void addActionListener(ActionListener l)
   {
	actionListener = AWTEventMulticaster.add(actionListener, l);
   }


  public void sendAction(String command)
  {
   if (actionListener != null)
	actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
  }


  public void componentShown(ComponentEvent e)
  {
    if(treeDir.getViewCount() < 1) treeDir.init();
    requestFocus();
 }

 public void componentHidden(ComponentEvent e) { }
 public void componentMoved(ComponentEvent e) { }
 public void componentResized(ComponentEvent e) { }

}



